'From Squeak 2.0 of May 22, 1998 on 1 June 1998 at 7:34:23 pm'!
"Change Set:		jhmRepaintFix
Date:			1 June 1998
Author:			John Maloney

Fixes a problem with re-painting an existing SketchMorph
when it is rotated. It was appearing in the upper left of the
World, rather than allowing painting in place."
!


!GradientFillMorph methodsFor: 'drawing' stamp: 'jm 6/1/1998 19:24'!
drawOn: aCanvas
	"Note that this could run about 4 times faster if we got hold of
	the canvas's port and just sent it copyBits with new coords and color"
 	| r colors step |
	super drawOn: aCanvas.
	(color isKindOf: Color) ifFalse: [^ self].  "An InfiniteForm, for example"
	(color = Color transparent) ifTrue: [^ self].  "Skip the gradient attempts, which will drop into debugger"
	color = fillColor2 ifTrue: [^ self].  "same color; no gradient"
	r _ self innerBounds intersect: aCanvas clipRect.
	colors _ self colorArrayForDepth: aCanvas depth.
	step _ self stepSize.
	gradientDirection = #vertical
		ifTrue:
		[r top to: r bottom-1 by: step do:
			[:y | aCanvas fillRectangle: (r left @ y corner: r right @ (y+step min: r bottom))
					color: (colors at: y - bounds top //step+1)]]
		ifFalse:
		[r left to: r right-1 by: step do:
			[:x | aCanvas fillRectangle: (x @ r top corner: (x+step min: r right) @ r bottom)
					color: (colors at: x - bounds left //step+1)]]! !


!HandMorph methodsFor: 'meta menu' stamp: 'jm 6/1/1998 19:15'!
makeNewDrawing
	"Make a new drawing in the standard playfield."

	| aPlayfield bnds |
	(aPlayfield _ self world playfield) ifNil: [aPlayfield _ self world].
	bnds _ aPlayfield paintingBoundsAround: aPlayfield boundsInWorld center.
	self makeNewDrawingInBounds: bnds pasteUpMorph: aPlayfield.
! !


!PasteUpMorph methodsFor: 'painting' stamp: 'jm 6/1/1998 19:16'!
makeNewDrawingWithin

	| bnds |
	bnds _ self paintingBoundsAround: self boundsInWorld center.
	self primaryHand makeNewDrawingInBounds: bnds pasteUpMorph: self.
! !

!PasteUpMorph methodsFor: 'painting' stamp: 'jm 6/1/1998 18:52'!
paintingBoundsAround: aPoint
	"Return a rectangle for painting centered on the given point. Both the argument point and the result rectangle are in world coordinates."

	| paintExtent maxPaintArea myBnds |
	paintExtent _ ScriptingSystem reasonablePaintingExtent.
	maxPaintArea _ paintExtent x * paintExtent y.
	myBnds _ self boundsInWorld.
	(myBnds area <= maxPaintArea) ifTrue: [^ myBnds].
	^ (aPoint - (paintExtent // 2) extent: paintExtent) intersect: myBnds
! !


!SketchMorph methodsFor: 'menu' stamp: 'jm 6/1/1998 19:04'!
editDrawingIn: aPasteUpMorph forBackground: forBackground

	| w sketchEditor oldRotation oldFwdDir bnds |
	w _ aPasteUpMorph world.
	w stopRunningAll; abandonAllHalos.
	w displayWorld.
	oldRotation _ rotationDegrees.
	oldFwdDir _ self forwardDirection.
	self rotationDegrees: 0.
	forBackground
		ifTrue: [bnds _ aPasteUpMorph boundsInWorld]
		ifFalse: [
			bnds _ self boundsInWorld.
			bnds _ (aPasteUpMorph paintingBoundsAround: bnds center) merge: bnds].

	sketchEditor _ SketchEditorMorph new.
	forBackground ifTrue: [sketchEditor setProperty: #background toValue: true].
	w addMorphFront: sketchEditor.
	sketchEditor initializeFor: self inBounds: bnds ofWorld: aPasteUpMorph world.
	self rotationDegrees: oldRotation.  "while drawing is still rotated. cancel leaves it right"
	sketchEditor
		afterNewPicDo: [:aForm :aRect |
			self form: aForm.
			self position: aRect origin.
			self forwardDirection: sketchEditor forwardDirection.
			self rotationDegrees: oldRotation + (sketchEditor forwardDirection - oldFwdDir).
				"add in any changes"
			self rotationStyle: sketchEditor rotationStyle.
			aPasteUpMorph playfield ifNotNil: "Show the right viewer"
				[self presenter drawingJustCompleted: self].
			forBackground ifTrue: [self goBehind].  "shouldn't be necessary"
			owner changed]
		ifNoBits: ["If no bits drawn.  Must keep old pic.  Can't have no picture"
			aPasteUpMorph standardPalette ifNotNil: [aPasteUpMorph standardPalette viewMorph: self]].

	sketchEditor changed.
! !


!WorldMorph methodsFor: 'initialization' stamp: 'jm 6/1/1998 19:26'!
initialize

	super initialize.
	color _ (Color r:0.937 g: 0.937 b: 0.937).
	fillColor2 _ color.
	hands _ Array new.
	self addHand: HandMorph new.
	viewBox _ canvas _ nil.
	damageRecorder _ DamageRecorder new.
	stepList _ OrderedCollection new.
	lastStepTime _ 0.
	model _ nil.
! !


PasteUpMorph removeSelector: #paintingBounds!
SketchEditorMorph removeSelector: #initializeFor:inPasteUpMorph:!
StandardScriptingSystem removeSelector: #maxPaintArea!
