'From Squeak 2.0 of May 22, 1998 on 2 June 1998 at 8:34:14 pm'!
"Change Set:		SqueakTweaks
Date:			2 June 1998
Author:			Dan Ingalls, John-Reed Maffeo

Installs a help-menu item for turning sound on and off.
Warns of redefining a class when not looking at it in the browser.

From John-Reed Maffeo:
A fix to pluggable buttons in FormMenuView."!


!Browser methodsFor: 'class functions' stamp: 'di 6/2/1998 13:58'!
defineClass: defString notifying: aController 
	"The receiver's textual content is a request to define a new class. The 
	source code is defString. If any errors occur in compilation, notify 
	aController."
	| oldClass class newClassName |
	oldClass _ self selectedClassOrMetaClass.
	newClassName _ (defString findTokens: Character separators) third copyWithout: $#.
	((oldClass isNil or: [oldClass name asString ~= newClassName])
		and: [Smalltalk includesKey: newClassName asSymbol]) ifTrue:
			["Attempting to define new class over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newClassName , ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newClassName size))
				ifFalse: [^ false]].
	oldClass ifNil: [oldClass _ Object].
	class _ oldClass subclassDefinerClass
				evaluate: defString
				notifying: aController
				logged: true.
	(class isKindOf: Behavior)
		ifTrue: 
			[self changed: #classList.
			self classListIndex: 
				(self classList indexOf: 
					((class isKindOf: Metaclass)
						ifTrue: [class soleInstance name]
						ifFalse: [class name])).
			self clearUserEditFlag; editClass.
			^true]
		ifFalse: [^false]! !


!FormMenuView methodsFor: 'private' stamp: 'jrm 6/1/1998 21:57'!
makeColorConnections: indexInterval

	| connector buttonCache button aSwitchView |
	connector _ Object new.  "a dummy model for connecting dependents"
	indexInterval do: [:index |
		buttonCache _ FormButtons at: index.
		buttonCache initialState = #true
			ifTrue: [button _ OneOnSwitch newOn]
			ifFalse: [button _ OneOnSwitch newOff].
		button onAction: [model changeTool: buttonCache value].
		button connection: connector.
		aSwitchView _ self makeViews: buttonCache for: button.
		aSwitchView
			borderWidthLeft: 1 right: 0 top: 1 bottom: 1;
			action: #turnOn].
	aSwitchView borderWidth: 1.
! !

!FormMenuView methodsFor: 'private' stamp: 'jrm 6/1/1998 21:56'!
makeConnections: indexInterval

	| connector buttonCache button aSwitchView |
	connector _ Object new.  "a dummy model for connecting dependents."
	indexInterval do: [:index |
		buttonCache _ FormButtons at: index.
		buttonCache initialState = #true
			ifTrue: [button _ OneOnSwitch newOn]
			ifFalse: [button _ OneOnSwitch newOff].
		button onAction: [model changeTool: buttonCache value].
		button connection: connector.
		aSwitchView _ self makeViews: buttonCache for: button.
		aSwitchView
			borderWidthLeft: 1 right: 0 top: 1 bottom: 1;
			action: #turnOn].
	aSwitchView borderWidth: 1.
! !


!PluggableButtonView methodsFor: 'accessing' stamp: 'jrm 6/1/1998 21:53'!
action: aSymbol 
	"Set actionSelector to be the action defined by aSymbol."

	actionSelector _ aSymbol
! !


!ScreenController methodsFor: 'menu messages' stamp: 'di 6/2/1998 15:37'!
soundOnOrOff
	Preferences setPreference: #disableSounds
		toValue: (Preferences valueOfFlag: #disableSounds) not! !

!ScreenController methodsFor: 'nested menus' stamp: 'di 6/2/1998 15:35'!
helpMenu
	"Answer the help menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'preferences...'
			'update code from server'
			'about this system...'
			'command-key help'
			'useful expressions'
			'set author initials...'
			'set desktop color...'
			'set display depth...'
			'full screen on'
			'full screen off' ) , 
			(Array with: ((Preferences valueOfFlag: #disableSounds)
							ifTrue: ['turn sound on']
							ifFalse: ['turn sound off'])) , 
		#(	'view GIF imports'
			'space left'
			'vm statistics' )
		lines: #(2 5 8 11)
		selections: #(editPreferences  absorbUpdatesFromServer aboutThisSystem openCommandKeyHelp openStandardWorkspace setAuthorInitials setDesktopColor setDisplayDepth fullScreenOn fullScreenOff soundOnOrOff viewGIFImports garbageCollect vmStatistics)
"
ScreenController new helpMenu startUp
"! !


!ScreenController class methodsFor: 'class initialization' stamp: 'di 6/2/1998 15:48'!
initialize
	"Initialize the screen menus.  Call this method to reset everything back to nil, so that the various menu retrieval methods will duly reinitialize them."

	"ScreenController initialize"
	ChangesMenu _ TopScreenMenu _ OpenMenu _ WindowMenu _ ProjectScreenMenu _ nil.! !


!ServerDirectory methodsFor: 'file directory' stamp: 'di 6/2/1998 13:33'!
entries
	"Return a collection of directory entries for the files and directories in this directory. Each entry is a five-element array: (<name><creationTime><modificationTime><dirFlag><fileSize>)."

	| ftpEntries thisYear tokens dateInSeconds longy |
	"We start with ftp directory entries of the form...
d---------   1 owner    group               0 Apr 27 22:01 blasttest
----------   1 owner    group           93812 Jul 21  1997 COMMAND.COM
    1        2   3           4                 5    6  7    8       9   -- token index"
	ftpEntries _ self getDirectory contentsOfEntireFile findTokens: FTPSocket crLf.

	thisYear _ Date today year.
	^ ftpEntries collect:
		[:ftpEntry | tokens _ ftpEntry findTokens: ' '.
		tokens size >= 9
		ifTrue:
		[tokens size > 9 ifTrue:
			[longy _ tokens at: 9.
			10 to: tokens size do: [:i | longy _ longy , ' ' , (tokens at: i)].
			tokens at: 9 put: longy].
		dateInSeconds _ ((tokens at: 8) includes: $:)
			ifTrue: ["Date has no year -- assume this year"
					(Date newDay: (tokens at: 7) asNumber
								month: (tokens at: 6)
								year: thisYear) asSeconds +
					(Time readFrom: (ReadStream on: (tokens at: 8))) asSeconds]
			ifFalse: [(Date newDay: (tokens at: 7) asNumber
								month: (tokens at: 6)
								year: (tokens at: 8) asNumber) asSeconds].
		Array with: (tokens at: 9)  "file name"
			with: dateInSeconds "creation date"
			with: dateInSeconds "modification time"
			with: tokens first first = $d "is-a-directory flag"
			with: tokens fifth asNumber "file size"]
		ifFalse:
		[nil]]
		thenSelect: [:entry | entry notNil]! !


ScreenController initialize!
"Postscript:
Executable statements after this comment quote..."
ScreenController removeClassVarName: 'HelpMenu'.
!

