'From Squeak 2.4b of April 23, 1999 on 9 July 1999 at 4:23:52 pm'!
"Change Set:		WnldRotInfo
Date:			9 July 1999
Author:			Jeff Pierce

Adds the actor rotation data to the Actor Info tab.  Also fixes the Actor Info tab to clear the selected actor on reset, and fixes reset so that prototype classes are created correctly for lights and cameras."!

WonderlandMorph subclass: #WonderlandActorData
	instanceVariableNames: 'nameMorph positionMorph rotationMorph parentMorph '
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants '
	category: 'Wonderland-Morphs'!

!Wonderland methodsFor: 'initialize-reset-release' stamp: 'jsp 7/9/1999 15:01'!
reset
	"Reset this Wonderland"

	| ground |

	"Reset the scheduler"
	myScheduler reset.

	"Reset the shared mesh and texture directories"
	sharedMeshDict _ Dictionary new.
	sharedTextureDict _ Dictionary new.

	"Reset the list of actor uniclasses"
	actorClassList do: [:aClass | aClass removeFromSystem ].
	actorClassList _ OrderedCollection new.

	"Initialize this Wonderland's shared namespace"
	myNamespace _ WonderlandConstants copy.

	"Rebuild the namespace"
	myNamespace at: 'scheduler' put: myScheduler.
	myNamespace at: 'w' put: self.

	"Create a new scene"
	sceneObject _ WonderlandScene newFor: self.
	myNamespace at: 'scene' put: sceneObject.

	"Recreate the default objects"
	ground _ self makeActor.
	ground setMesh: (WonderlandConstants at: 'groundMesh').
	ground setTexturePointer: (WonderlandConstants at: 'groundTexture').
	ground becomePart.
	ground setName: 'ground'.
	ground setColor: {0.2902. 0.8000. 0.0000} duration: rightNow.
	myNamespace at: 'ground' put: ground.

	"Re-initialize the light list and create a default light"
	lightList _ OrderedCollection new.
	self makeLight.
	lightList last moveTo: {0. 2. 0} duration: rightNow.

	"Wipe out the existing cameras"
	cameraList do: [:camera | camera release].
	
	"Recreate the default camera"
	cameraList _ OrderedCollection new.
	defaultCamera _ self makeCamera.

	"Reset the script editor's namespace"
	scriptEditor resetNamespace.

	"Update the actor browser"
	scriptEditor updateActorBrowser.

	"Reset the undo stack"
	myUndoStack reset.


! !


!WonderlandActorData methodsFor: 'initialization' stamp: 'jsp 7/9/1999 16:23'!
initialize
	"Initialize this morph by setting up the various submorphs to be able to show data about a WonderlandActor"

	| aMorph |

	super initialize.
	self color: (Color r: 0.815 g: 0.972 b: 0.878).
	self bounds: (0@0 corner: 300@120).

	nameMorph _ TextMorph new.
	nameMorph contents: 'Name: (No actor selected)'.
	nameMorph topLeft: 10@10.
	self addMorph: nameMorph.

	parentMorph _ TextMorph new.
	parentMorph contents: 'Parent: '.
	parentMorph topLeft: 10@25.
	self addMorph: parentMorph.

	aMorph _ TextMorph new.
	aMorph contents: 'Position relative to parent'.
	aMorph topLeft: 10@55.
	self addMorph: aMorph.

	positionMorph _ TextMorph new.
	positionMorph contents: ((Character tab) asString) , 'Right:    Up:    Forward: '.
	positionMorph topLeft: 10@70.
	self addMorph: positionMorph.

	aMorph _ TextMorph new.
	aMorph contents: 'Rotation about'.
	aMorph topLeft: 10@100.
	self addMorph: aMorph.

	rotationMorph _ TextMorph new.
	rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
				((Character cr) asString) , ((Character tab) asString) , 'Up-Down axis: ',
				((Character cr) asString) , ((Character tab) asString) , 'Forward-Back axis: '.
	rotationMorph topLeft: 10@115.
	self addMorph: rotationMorph.

! !

!WonderlandActorData methodsFor: 'setting actor' stamp: 'jsp 7/9/1999 16:20'!
setActor: anActor
	"Set the actor whose data this morph should show and update the displayed data."

	| position rotation |

	(anActor isNil)
		ifTrue: [
				nameMorph contents: 'Name: (No actor selected)'.
				parentMorph contents: 'Parent: '.	
				positionMorph contents: ((Character tab) asString) , 'Right:    Up:    Forward: '.
				rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
					((Character cr) asString) , ((Character tab) asString) , 'Up-Down axis: ',
					((Character cr) asString) , ((Character tab) asString) , 'Forward-Back axis: '.
				]

		ifFalse: [
				nameMorph contents: ('Name: ' , (anActor getName)).

				(anActor isKindOf: WonderlandActor)
					ifTrue: [
							parentMorph contents: ('Parent: ' , (anActor getParent getName)).

							position _ anActor getPosition.
							position at: 1 put: ((((position at: 1) * 1000) rounded / 1000.0) asString).
							position at: 2 put: ((((position at: 2) * 1000) rounded / 1000.0) asString).
							position at: 3 put: ((((position at: 3) * 1000) rounded / 1000.0) asString).
							positionMorph contents: ((Character tab) asString) , ('Right: ',
								(position at: 1) , '  Up: ', (position at: 2) , '  Forward: ' ,
								(position at: 3)).

							rotation _ anActor getAngles.
							rotation at: 1 put: ((((rotation at: 1) * 1000) rounded / 1000.0) asString).
							rotation at: 2 put: ((((rotation at: 2) * 1000) rounded / 1000.0) asString).
							rotation at: 3 put: ((((rotation at: 3) * 1000) rounded / 1000.0) asString).
							rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
								(rotation at: 1) , ((Character cr) asString) ,
								((Character tab) asString) , 'Up-Down axis: ' , (rotation at: 2) , 
								((Character cr) asString) , ((Character tab) asString) ,
								'Forward-Back axis: ' , (rotation at: 3).

							]
					ifFalse: [
							parentMorph contents: 'Parent: None'.	
							positionMorph contents: ((Character tab) asString) ,
									'Right:    Up:    Forward: '.
							rotationMorph contents: ((Character tab) asString) , 'Left-Right axis: ' ,
								((Character cr) asString) , ((Character tab) asString) ,
								'Up-Down axis: ', ((Character cr) asString) ,
								((Character tab) asString) , 'Forward-Back axis: '.
							]

				].



! !


!WonderlandEditorTabs methodsFor: 'accessing' stamp: 'jsp 7/9/1999 15:03'!
resetNamespace
	"Reset the namespace used by the script editor"

	myScriptEditor setBindings: (myWonderland getNamespace).
	myActorViewer setSelectedActor: nil.

! !


