'From Squeak 2.4c of May 10, 1999 on 27 July 1999 at 12:57:15 pm'!
"Change Set:		griddingCleanup-sw
Date:			27 July 1999
Author:			Scott Wallace

Some improvements to morphic gridding:

*  The command 'turn gridding on' just turns gridding on, using the current grid size.

*  The new command 'set grid size' allows you to specify (or confirm) the grid size.

*  When specifying a new grid size, you can respond with a simple integer and it will be used to grid both dimensions (formerly you had to respond with a Point, or things would go sour.)

*  When specifying a new grid size, the input is checked to make certain that it is of appropriate type and in an appropriate range (1 .. 36) before it is truly accepted. 

(NB:  since the Hand grids its input points, large grid sizes can quickly lead to an unusable UI -- for example you may not be able to select certain items in menus, including perhaps the items necessary to reset the grid size to a more plausible value!!.  36 is perhaps an over-generous max grid size.)

*  Use of the 'set grid size' will turn gridding on if it were previously off; used in this way, it thus behaves the same as the original turn gridding on command."!


!HandMorph methodsFor: 'world menu' stamp: 'sw 7/27/1999 12:36'!
helpMenu
	"Build the help menu for the world."
	| menu screenCtrl |
	screenCtrl _ ScreenController new.
	menu _ (MenuMorph entitled: 'help...') defaultTarget: self.
	menu addStayUpItem.
	menu add: 'about this system...' target: Smalltalk action: #aboutThisSystem.
	menu balloonTextForLastItem: 'current version information.'.
	menu add: 'update code from server' action: #absorbUpdatesFromServer.
	menu balloonTextForLastItem: 'load latest code updates via the internet'.
	menu add: 'preferences...' target: Preferences action: #openPreferencesInspector.
	menu balloonTextForLastItem: 'view and change various options.'.
	menu addLine.

	menu add: 'command-key help' target: Utilities action: #openCommandKeyHelp.
	menu balloonTextForLastItem: 'summary of keyboard shortcuts.'.
	menu add: 'world menu help' target: self action: #worldMenuHelp.
	menu balloonTextForLastItem: 'helps find menu items buried in submenus.'.
	"menu add: 'info about flaps' target: Utilities action: #explainFlaps."
	menu balloonTextForLastItem: 'describes how to enable and use flaps.'.
	menu add: 'font size summary' target: Utilities action: #fontSizeSummary.
	menu balloonTextForLastItem: 'summary of names and sizes of available fonts.'.
	menu add: 'useful expressions' target: Utilities action: #openStandardWorkspace.
	menu balloonTextForLastItem: 'a window full of useful expressions.'.
	menu add: 'view graphical imports' target: Smalltalk action: #viewImageImports.
	menu balloonTextForLastItem: 'lets you look at graphical imports.'.
	menu addLine.

	menu addUpdating: #gridOnString action: #setGridding.
	menu balloonTextForLastItem: 'turn gridding on or off.'.
	menu add: 'set grid size...' action: #setGridSize.
	menu balloonTextForLastItem: 'specify the size of the grid.'.

	menu add: 'telemorphic...' action: #remoteDo.
	menu balloonTextForLastItem: 'commands for doing multi-machine "telemorphic" experiments'.
	menu addUpdating: #soundEnablingString target: Preferences action: #toggleSoundEnabling.
	menu balloonTextForLastItem: 'turning sound off will completely disable Squeak''s use of sound.'.
	menu add: 'definition for...' target: Utilities action: #lookUpDefinition.
	menu balloonTextForLastItem: 'if connected to the internet, use this to look up the definition of an English word.'.
	menu addLine.

	menu add: 'set author initials...' target: screenCtrl action: #setAuthorInitials.
	menu balloonTextForLastItem: 'supply initials to be used to identify the author of code and other content.'.
	menu add: 'vm statistics' target: screenCtrl action: #vmStatistics.
	menu balloonTextForLastItem: 'obtain some intriguing data about the vm.'.
	menu add: 'space left' target: screenCtrl action: #garbageCollect.
	menu balloonTextForLastItem: 'perform a full garbage-collection and report how many bytes of space remain in the image.'.
	^ menu
! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 7/27/1999 12:49'!
setGridSize

	| response result |
	response _
		FillInTheBlank
			request: 'New grid size: '
			initialAnswer: grid printString.
	response isEmpty ifTrue: [^ self].
	result _ Compiler evaluate: response.
	gridOn _ true.  "in case it was off -- otherwise this cmd might be upsetting!!"
	((result isKindOf: Point) and: [(result > (0@0)) & (result < (37 @ 37))]) ifTrue:
		[^ grid _ result].
	((result isKindOf: SmallInteger) and: [(result > 0) & (result < 37)]) ifTrue:
		[^ grid _ result @ result].
	self inform: 'Must be a Point or an Integer with
coordinates between 1 & 36'	
! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 7/27/1999 12:44'!
setGridding
	gridOn _ (gridOn == true) not! !


"Postscript:"
Utilities replaceMenuFlap.
!

