'From Squeak 2.0 of May 22, 1998 on 30 June 1998 at 12:14:23 pm'!
"Change Set:		SimulationGuard
Date:			5 June 1998
Author:			Andreas Raab

This change set should fix most of the crashes during simulation
by using a guard primitive which is installed in all method that
cannot possibly be simulated."!


!ContextPart methodsFor: 'private' stamp: 'ar 6/5/1998 21:57'!
doPrimitive: primitiveIndex receiver: receiver args: arguments 
	"Simulate a primitive method whose index is primitiveIndex.  The
	simulated receiver and arguments are given as arguments to this message."
"
	NOTE: In order for perform:WithArguments: to work reliably here,
	this method must be forced to invoke a large context.  This is done
	by adding extra temps until the following expression evaluates as true:
		(ContextPart compiledMethodAt: #doPrimitive:receiver:args:) frameSize > 20
"
	| value t1 t2 t3 |
	<primitive: 19> "Simulation guard"
	"If successful, push result and return resuming context,
		else ^ #simulatorFail"
	(primitiveIndex = 19) ifTrue:[
		Debugger 
			openContext: self
			label:'Code simulation error'
			contents: self shortStack].

	(primitiveIndex = 80 and: [receiver isKindOf: ContextPart])
		ifTrue: [^self push: 
					((BlockContext new: receiver size)
						home: receiver home
						startpc: pc + 2
						nargs: (arguments at: 1))].
	(primitiveIndex = 81 and: [receiver isMemberOf: BlockContext])
		ifTrue: [^receiver pushArgs: arguments from: self].
	(primitiveIndex = 83 or:[primitiveIndex = 84])
		ifTrue: [^ self send: arguments first to: receiver
					with: arguments allButFirst
					super: false].
	arguments size > 6 ifTrue: [^#simulatorFail].
	value _ receiver tryPrimitive: primitiveIndex withArgs: arguments.
	value == #simulatorFail
		ifTrue: [^ #simulatorFail]
		ifFalse: [^ self push: value]! !


!BlockContext methodsFor: 'scheduling' stamp: 'ar 6/5/1998 21:44'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."
	<primitive: 19> "Simulation guard"
	^Process
		forContext: 
			[self value.
			Processor terminateActive]
		priority: Processor activePriority! !

!BlockContext methodsFor: 'scheduling' stamp: 'ar 6/5/1998 21:44'!
newProcessWith: anArray 
	"Answer a Process running the code in the receiver. The receiver's block 
	arguments are bound to the contents of the argument, anArray. The 
	process is not scheduled."
	<primitive: 19> "Simulation guard"
	^Process
		forContext: 
			[self valueWithArguments: anArray.
			Processor terminateActive]
		priority: Processor activePriority! !


!ControlManager methodsFor: 'accessing' stamp: 'ar 6/5/1998 21:49'!
activeController: aController 
	"Set aController to be the currently active controller. Give the user 
	control in it."
	<primitive: 19> "Simulation guard"
	activeController _ aController.
	(activeController == screenController)
		ifFalse: [self promote: activeController].
	activeControllerProcess _ 
			[activeController startUp.
			self searchForActiveController] newProcess.
	activeControllerProcess priority: Processor userSchedulingPriority.
	activeControllerProcess resume! !

!ControlManager methodsFor: 'scheduling' stamp: 'ar 6/5/1998 21:48'!
scheduleActive: aController 
	"Make aController be scheduled as the active controller. Presumably the 
	active scheduling process asked to schedule this controller and that a 
	new process associated this controller takes control. So this is the last act 
	of the active scheduling process."
	<primitive: 19> "Simulation guard"
	self scheduleActiveNoTerminate: aController.
	Processor terminateActive! !


!Debugger class methodsFor: 'opening' stamp: 'di 6/30/1998 12:12'!
openContext: aContext label: aString contents: contentsString
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."
	| f |
	<primitive: 19> "Simulation guard"
	ErrorRecursion ifTrue:
		[ErrorRecursion _ false.
		self primitiveError: aString].
	false ifTrue:  "Set to true for log to file"
		[f _ FileStream fileNamed: 'SqueakDebug.log'.
		f nextPutAll: aString; cr.
		aContext stack do: [:ctxt | f print: ctxt; cr].
		f close.
		self primitiveError: aString].
	ErrorRecursion _ true.
	(Debugger context: aContext)
		openNotifierContents: contentsString
		label: aString.
	ErrorRecursion _ false.
	Processor activeProcess suspend.
! !

!Debugger class methodsFor: 'opening' stamp: 'di 6/30/1998 12:13'!
openInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."
	| debugger |
	<primitive: 19> "Simulation guard"
	debugger _ self new.
	debugger
		process: interruptedProcess
		controller: ((Smalltalk isMorphic not
					and: [ScheduledControllers activeControllerProcess == interruptedProcess])
						ifTrue: [ScheduledControllers activeController])
		context: interruptedProcess suspendedContext.
	debugger externalInterrupt: true.
	^ debugger
		openNotifierContents: debugger interruptedContext shortStack
		label: aString
! !


