'From Squeak 2.1 of June 30, 1998 on 5 July 1998 at 9:10:25 pm'!
"Change Set:		DragTweaks
Date:			5 July 1998
Author:			Dan Ingalls

Restored the brown handle to morphic command halo, allowing a morph to be dragged within its owner's bounds without extracting it or changing its front-to-back order.
Put in a feature that allows this constrained drag to be accessed merely by dragging more than 5 pixels with the cmd-key held down.

Made handle size parametric in existing halo code.
Adjusted placment logic of a few other command handles to work better on very small morphs.
Fix a performance bug that repainted everything between 0@0 and a halo at pop-up time.
"!

!Morph methodsFor: 'geometry' stamp: 'di 7/5/1998 20:57'!
setConstrainedPositionFrom: aPoint
	"Change the position of this morph and and all of its submorphs to aPoint, but don't let me go outside my owner's bounds."

	| trialRect delta |
	trialRect _ aPoint extent: self bounds extent.
	delta _ trialRect amountToTranslateWithin: owner bounds.
	self position: aPoint + delta
! !

!Morph methodsFor: 'halos and balloon help' stamp: 'di 7/5/1998 12:52'!
addHalo
	| halo |
	halo _ HaloMorph new bounds: self bounds.
	self world addMorphFront: halo.
	halo target: self.
	halo startStepping.
! !

!Morph methodsFor: 'jun98 additions' stamp: 'di 7/5/1998 12:08'!
addFontHandlesTo: aHalo box: box
	"Add handles for controlling font, style, and emphasis choices to the halo"
	| s |
	s _ aHalo handleSize.
	aHalo addHandleAt: (box bottomLeft + ((s+2)@0))
		color: Color lightGreen on: #mouseDown send: #chooseFont to: self.
	aHalo addHandleAt: (box bottomLeft + ((s+2*2)@0))
		color: Color lightRed on: #mouseDown send: #chooseStyle to: self.
	aHalo addHandleAt: (box bottomRight - ((s+2)@0))
		color: Color lightBrown darker on: #mouseDown send: #chooseEmphasis to: self.

! !


!HaloMorph methodsFor: 'private' stamp: 'di 7/5/1998 11:55'!
addHandleAt: aPoint color: aColor on: eventName send: selector to: recipient
	"Add a handle centered at the given point with the given color, and set it up to respond to the given event by sending the given selector to the given recipient.  Return the handle."
	| handle |
	handle _ EllipseMorph
		newBounds: (Rectangle center: aPoint extent: self handleSize asPoint)
		color: aColor.
	self addMorph: handle.
	handle on: #mouseUp send: #endInteraction to: self.
	handle on: eventName send: selector to: recipient.
	handle setBalloonText: (target balloonHelpTextForHandle: handle).
	^ handle
! !

!HaloMorph methodsFor: 'private' stamp: 'di 7/5/1998 12:45'!
addHandles
	| box s |
	target isWorldMorph ifTrue: [^ self addHandlesForWorldHalos].
	self removeAllMorphs.  "remove old handles, if any"
	s _ self handleSize.
	self bounds: target fullBoundsInWorld.  "update my size"
	box _ (self fullBounds expandBy: s+1)
			intersect: (self world bounds insetBy: 5@5).

	self addHandleAt: box topLeft color: Color red
		on: #mouseDown send: #doMenu:with: to: self.

	self addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #tearOffTile to: innerTarget.

	self addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color transparent
		on: #mouseDown send: #dismiss to: self.
	self addHandleAt: (box leftCenter) color: Color cyan
		on: #mouseDown send: #openViewerForArgument to: innerTarget.

	self addHandleAt: box topCenter color: Color black
		on: #mouseDown send: #doGrab:with: to: self.

	(self addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown
		on: #mouseDown send: #startDrag:with: to: self)
		on: #mouseStillDown send: #doDrag:with: to: self.

	self addHandleAt: box topRight color: Color green
		on: #mouseDown send: #doDup:with: to: self.

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue
			on: #mouseDown send: #mouseDownOnHelpHandle: to: innerTarget)
			on: #mouseUp send: #deleteBalloon to: innerTarget].

	(self addHandleAt: box bottomLeft color: Color blue
		on: #mouseDown send: #startRot:with: to: self)
		on: #mouseStillDown send: #doRot:with: to: self.

	target isFlexMorph
		ifTrue: [(self addHandleAt: box bottomRight color: Color lightOrange
				on: #mouseDown send: #startScale:with: to: self)
				on: #mouseStillDown send: #doScale:with: to: self]
		ifFalse: [(self addHandleAt: box bottomRight color: Color yellow
				on: #mouseDown send: #startGrow:with: to: self)
				on: #mouseStillDown send: #doGrow:with: to: self].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: box string: target externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !

!HaloMorph methodsFor: 'private' stamp: 'di 7/5/1998 12:34'!
addHandlesForWorldHalos
	| box s |
	self removeAllMorphs.  "remove old handles, if any"
	self bounds: target bounds.
	box _ self world bounds insetBy: 9.
	s _ self handleSize.

	self addHandleAt: box topLeft color: Color red on: #mouseDown send: #doMenu:with: to: self.
	self addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #tearOffTile to: target.
	self addHandleAt: box leftCenter color: Color cyan
		on: #mouseDown send: #openViewerForArgument to: target.

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue
			on: #mouseDown send: #mouseDownOnHelpHandle: to: target)
			on: #mouseUp send: #deleteBalloon to: target].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: (box insetBy: (0@0 corner: 0@10)) string: innerTarget externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !

!HaloMorph methodsFor: 'private' stamp: 'di 7/5/1998 11:54'!
handleSize
	^ 16! !


!HandMorph methodsFor: 'event dispatching' stamp: 'di 7/5/1998 14:05'!
processEvents
	"Process user input events from the local input devices."

	| griddedPoint evt currentExtent |
	griddedPoint _ Sensor cursorPoint - owner viewBox topLeft.
	gridOn ifTrue: [griddedPoint _ griddedPoint grid: grid].

	evt _ MorphicEvent new
		setMousePoint: griddedPoint
		buttons: Sensor primMouseButtons
		lastEvent: lastEvent
		hand: self.

	remoteConnections size > 0 ifTrue: [
		currentExtent _ self worldBounds extent.
		lastWorldExtent ~= currentExtent ifTrue: [
			self transmitEvent: (MorphicEvent newWorldExtent: currentExtent).
			lastWorldExtent _ currentExtent].
		self transmitEvent: evt].

	(evt anyButtonPressed and:
	 [evt controlKeyPressed and:
	 [lastEvent anyButtonPressed not]]) ifTrue:
		[eventTransform _ MorphicTransform identity.
		lastEvent _ evt.
		^ self invokeMetaMenu: evt].

		evt blueButtonPressed ifTrue:
			[lastEvent blueButtonPressed 
				ifTrue: [^ self specialDrag: evt]
				ifFalse: [eventTransform _ MorphicTransform identity.
						lastEvent _ evt.
						^ self specialGesture: evt]].

	lastEvent _ evt.
	self handleEvent: evt.

	[Sensor keyboardPressed] whileTrue: [
		evt _ MorphicEvent new
			setKeyValue: Sensor keyboard asciiValue
			mousePoint: griddedPoint
			buttons: Sensor primMouseButtons
			hand: self.
		lastEvent _ evt.
		self handleEvent: evt.
		remoteConnections size > 0 ifTrue: [self transmitEvent: evt]].
! !

!HandMorph methodsFor: 'special gestures' stamp: 'di 7/5/1998 21:03'!
specialDrag: evt
	"Special drag (cmd-mouse on the Macintosh) allows a morph to be dragged without grabbing it and thus without removing it from its owner or changing its z-order." 
	| halo |
	self position ~= evt cursorPoint
		ifTrue: [self position: evt cursorPoint].
	mouseDownMorph
		ifNil:    ["Waiting for more than 5 pixels move to start drag"
				(self position dist: targetOffset) > 5
					ifTrue: [(halo _ self world haloMorphOrNil) ifNil: [^ self].
							mouseDownMorph _ halo target.
							targetOffset _ targetOffset - mouseDownMorph position.
							halo removeAllHandlesBut: nil]]
		ifNotNil: [mouseDownMorph setConstrainedPositionFrom: self position - targetOffset].! !

!HandMorph methodsFor: 'special gestures' stamp: 'di 7/5/1998 21:03'!
specialGesture: evt
	"Special gestures (cmd-mouse on the Macintosh) allow a mouse-sensitive morph to be grabbed or bring up a halo for the morph."
	"Summary:
		Cmd-mouse			pop up halo
		Cmd-shift-mouse		grab morph"

	mouseDownMorph _ nil.

	"if carrying morphs, just drop them"
	self hasSubmorphs ifTrue: [^ self dropMorphsEvent: evt].

	targetOffset _ menuTargetOffset _ self position.
	argument _ self argumentOrNil.
	evt shiftPressed
		ifTrue: [argument ifNotNil: [self grabMorph]]
		ifFalse: [self popUpHaloFromClick: evt].
! !


!RemoteHandMorph methodsFor: 'other' stamp: 'di 7/5/1998 21:07'!
processEvents
	"Process user input events from the remote input devices."

	| evt |
	evt _ self getNextRemoteEvent.
	[evt ~~ nil] whileTrue: [
		evt type == #worldExtent ifTrue: [
			remoteWorldExtent _ evt cursorPoint.
			^ self].

		(evt anyButtonPressed and:
		 [evt controlKeyPressed and:
		 [lastEvent anyButtonPressed not]]) ifTrue:
			[eventTransform _ MorphicTransform identity.
			lastEvent _ evt.
			^ self invokeMetaMenu: evt].

		evt blueButtonPressed ifTrue:
			[lastEvent blueButtonPressed 
				ifTrue: [^ self specialDrag: evt]
				ifFalse: [eventTransform _ MorphicTransform identity.
						lastEvent _ evt.
						^ self specialGesture: evt]].

		self handleEvent: evt.
		lastEvent _ evt.
		evt _ self getNextRemoteEvent]
! !


HandMorph removeSelector: #doSpecialDrag:!
