'From Squeak 2.5 of August 6, 1999 [latest update: #1508] on 29 September 1999 at 3:07:58 pm'!
"Change Set:		sysWindowMenu-sw
Date:			29 September 1999
Author:			Scott Wallace

Adds a menu to the title bar of system windows, affording easy access to frequently-needed functions such as renaming, sending to back, and making unclosable (see below).

Makes it possible for a system window to be designated as unclosable, in which case it will not sport a close-box (who among us has not inadvertently dismissed an important window because we happened to touch its close box?), and will not be closed by the wholesale window-menu commands 'delete unchanged windows'.

Adds a command to the window menu making it easy to flip-flop between two windows that are stacked above one another.

Note that a window's model is given the opportunity to enhance the window menu with its personal items if it so chooses."!

MorphicModel subclass: #SystemWindow
	instanceVariableNames: 'labelString stripes label closeBox collapseBox activeOnlyOnTop paneMorphs paneRects collapsedFrame fullFrame isCollapsed menuBox mustNotClose labelWidgetAllowance '
	classVariableNames: 'TopWindow '
	poolDictionaries: ''
	category: 'Morphic-Windows'!

!Model methodsFor: 'menus' stamp: 'sw 9/29/1999 10:24'!
addModelItemsToWindowMenu: aMenu
	"aMenu is being constructed to be presented to the user in response to the user's pressing on the menu widget in the title bar of a morphic window.  Here, the model is given the opportunity to add any model-specific items to the menu, whose default target is the SystemWindow itself."! !


!StandardScriptingSystem methodsFor: 'form dictionary' stamp: 'sw 5/12/1999 10:09'!
installSolidMenuForm
	"ScriptingSystem installSolidMenuForm"
	self saveForm:
		(Form extent: 14@16 depth: 16
	fromArray: #( 1 0 0 0 0 0 0 65537 65536 0 0 0 65537 0 65537 65537 65537 65537 65537 65537 65536 65537 65537 65537 65537 65537 1600061441 65536 65537 1600085855 1600085855 1600085855 1600085855 1600061441 65536 65537 1600085855 65537 65537 65537 65537 65536 65537 1600085855 65537 65537 65537 1600061441 65536 65537 1600085855 1600085855 1600085855 1600085855 1600085855 65537 65537 1600085855 65537 65537 65537 1600085855 65537 65537 1600085855 1600061441 65537 65537 89951 65537 65537 1600085855 1600085855 1600085855 1600085855 1600085855 65537 65537 1600085855 1600061441 65537 65537 65537 65537 65537 1600085855 65537 65537 65537 65536 65537 65537 65537 65537 65537 65537 65537 65537 1 65537 65537 65537 65537 65537 65536 0 65536 0 0 0 0 0) offset: 0@0)
		atKey: 'SolidMenu'! !


!SystemWindow methodsFor: 'initialization' stamp: 'sw 9/28/1999 13:57'!
addCloseBox
	self addMorph: (closeBox _ SimpleButtonMorph new borderWidth: 0;
			label: 'X' font: Preferences fontForScriptorButtons; color: Color transparent;
			actionSelector: #delete; target: self; extent: 16@16)! !

!SystemWindow methodsFor: 'initialization' stamp: 'sw 9/29/1999 14:59'!
addMenuControl
	self addMorph: (menuBox _ IconicButton new borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: 'TinyMenu'); color: Color transparent; 
			actWhen: #buttonDown;
			actionSelector: #offerWindowMenu; target: self;
			setBalloonText: 'window menu')

"NB: for the moment, we always supply balloon help for this control, until people get used to it; eventually, we mays switch to showing this balloon help only in novice mode, as we do for the other standard window controls."! !

!SystemWindow methodsFor: 'initialization' stamp: 'sw 9/29/1999 14:50'!
initialize
	| aFont |
	super initialize.
	labelString ifNil: [labelString _ 'Untitled Window'].
	isCollapsed _ false.
	activeOnlyOnTop _ true.
	paneMorphs _ Array new.
	paneRects _ Array new.
	borderColor _ #raised.
	borderWidth _ 1.
	color _ Color black.
	aFont _ Preferences fontForScriptorButtons.
	stripes _ Array with: (RectangleMorph newBounds: bounds)  "see extent:"
				with: (RectangleMorph newBounds: bounds).
	self addMorph: (stripes first borderWidth: 1).
	self addMorph: (stripes second borderWidth: 2).
	self addMorph: (label _ StringMorph new contents: labelString;
			font: (TextStyle default fontAt: 2) emphasis: 1).
	self setLabelWidgetAllowance.
	self addCloseBox.
	self addMenuControl.
	self addMorph: (collapseBox _ SimpleButtonMorph new borderWidth: 0;
			label: 'O' font: aFont; color: Color transparent;
			actionSelector: #collapseOrExpand; target: self; extent: 16@16).
	Preferences noviceMode ifTrue:
		[closeBox ifNotNil: [closeBox setBalloonText: 'close window'].
		menuBox ifNotNil: [menuBox setBalloonText: 'window menu'].
		collapseBox ifNotNil: [collapseBox setBalloonText: 'collapse/expand window']].
	self on: #mouseEnter send: #spawnReframeHandle: to: self.
	self on: #mouseLeave send: #spawnReframeHandle: to: self.
	label on: #mouseDown send: #relabelEvent: to: self.
	self extent: 300@200.
	mustNotClose _ false! !

!SystemWindow methodsFor: 'geometry' stamp: 'sw 9/29/1999 07:25'!
extent: newExtent
	| inner labelRect |
	isCollapsed
		ifTrue: [super extent: newExtent x @ (self labelHeight + 2)]
		ifFalse: [super extent: newExtent].
	inner _ self innerBounds.
	labelRect _ self labelRect.

	stripes first bounds: (labelRect insetBy: 1).
	stripes second bounds: (labelRect insetBy: 3).
	self setStripeColorsFrom: self paneColor.
	closeBox ifNotNil:
		[closeBox align: closeBox topLeft with: inner topLeft + (4@0)].
	menuBox ifNotNil:
		[menuBox align: menuBox topLeft with: (inner topLeft + (19@1))].
	collapseBox align: collapseBox topRight with: inner topRight - (4@0).
	label fitContents; setWidth: (label width min: bounds width - self labelWidgetAllowance).
	label align: label bounds topCenter with: inner topCenter.
	self setBoundsOfPaneMorphs.
	isCollapsed
		ifTrue: [collapsedFrame _ self bounds]
		ifFalse: [fullFrame _ self bounds].
! !

!SystemWindow methodsFor: 'geometry' stamp: 'sw 9/28/1999 11:02'!
removeMenuBox
	menuBox ifNotNil:
		[menuBox delete.
		menuBox _ nil].
! !

!SystemWindow methodsFor: 'label' stamp: 'sw 9/28/1999 13:39'!
labelHeight
	^ label height + 1 max: collapseBox height! !

!SystemWindow methodsFor: 'label' stamp: 'sw 9/29/1999 07:22'!
labelWidgetAllowance
	^ labelWidgetAllowance ifNil: [self setLabelWidgetAllowance]! !

!SystemWindow methodsFor: 'label' stamp: 'sw 9/29/1999 08:00'!
relabel
	| newLabel |
	newLabel _ FillInTheBlank 
		request: 'New titel for this window'
		initialAnswer: labelString.
	newLabel isEmpty ifTrue: [^self].
	(model windowReqNewLabel: newLabel)
		ifTrue: [self setLabel: newLabel]! !

!SystemWindow methodsFor: 'label' stamp: 'sw 9/29/1999 07:25'!
setLabel: aString
	labelString _ aString.
	label ifNil: [^ self].
	label contents: aString.
	self labelWidgetAllowance.  "Sets it if not already"
	self isCollapsed
		ifTrue: [self extent: (label width + labelWidgetAllowance) @ (self labelHeight + 2)]
		ifFalse: [label fitContents; setWidth: (label width min: bounds width - labelWidgetAllowance).
				label align: label bounds topCenter with: bounds topCenter + (0@borderWidth).
				collapsedFrame ifNotNil:
					[collapsedFrame _ collapsedFrame withWidth: label width + labelWidgetAllowance]]! !

!SystemWindow methodsFor: 'label' stamp: 'sw 9/29/1999 07:22'!
setLabelWidgetAllowance
	^ labelWidgetAllowance _ 75! !

!SystemWindow methodsFor: 'open/close' stamp: 'sw 9/28/1999 13:33'!
delete
	| thisWorld sketchEditor aPaintBox |
	self mustNotClose ifTrue: [^ self].
	model okToChange ifFalse: [^ self].
	thisWorld _ self world.
	sketchEditor _ self extantSketchEditor.
	super delete.
	model windowIsClosing; release.
	model _ nil.
	sketchEditor ifNotNil:
		[sketchEditor deleteSelfAndSubordinates.
		thisWorld notNil ifTrue:
			[(aPaintBox _ thisWorld paintBoxOrNil) ifNotNil: [aPaintBox delete]]].
		
	SystemWindow noteTopWindowIn: thisWorld.
! !

!SystemWindow methodsFor: 'open/close' stamp: 'sw 9/28/1999 13:32'!
mustNotClose
	^ mustNotClose == true! !

!SystemWindow methodsFor: 'resize/collapse' stamp: 'sw 9/29/1999 07:24'!
collapseOrExpand
	isCollapsed
	ifTrue:
		["Expand -- restore panes to morphics structure"
		isCollapsed _ false.
		super bounds: fullFrame.
		paneMorphs reverseDo: [:m | self addMorph: m].
		self activate "-- mainly for findWindow"]
	ifFalse:
		["Collapse -- remove panes from morphics structure"
		isCollapsed _ true.
		paneMorphs do: [:m | m delete; releaseCachedState].
		model modelSleep.
		collapsedFrame
			ifNil:	[self extent: (label width + self labelWidgetAllowance) @ (self labelHeight + 2).
					self position: (RealEstateAgent assignCollapsePointFor: self)]
			ifNotNil: [super bounds: collapsedFrame]]! !

!SystemWindow methodsFor: 'menu' stamp: 'sw 9/28/1999 13:53'!
makeClosable
	mustNotClose _ false.
	closeBox ifNil:
		[self addCloseBox.
		self extent: self extent]! !

!SystemWindow methodsFor: 'menu' stamp: 'sw 9/29/1999 10:46'!
makeSecondTopmost
	| aWorld nextWindow |
	aWorld _ self world.
	nextWindow _ aWorld submorphs detect:
		[:m | (m isKindOf: SystemWindow) and:  [m ~~ self]] ifNone: [^ self].
	nextWindow activate.
	aWorld addMorph: self behind: nextWindow
! !

!SystemWindow methodsFor: 'menu' stamp: 'sw 9/28/1999 13:52'!
makeUnclosable
	mustNotClose _ true.
	closeBox ifNotNil:
		[closeBox delete.
		closeBox _ nil]! !

!SystemWindow methodsFor: 'menu' stamp: 'sw 9/29/1999 10:47'!
offerWindowMenu
	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu add: 'change title...' action: #relabel.
	aMenu addLine.
	aMenu add: 'send to back' action: #sendToBack.
	aMenu add: 'make next-to-topmost' action: #makeSecondTopmost.
	aMenu addLine.
	self mustNotClose
		ifFalse:
			[aMenu add: 'make unclosable' action: #makeUnclosable]
		ifTrue:
			[aMenu add: 'make closable' action: #makeClosable].
	model ifNotNil:
		[model addModelItemsToWindowMenu: aMenu].
	aMenu popUpAt: self cursorPoint event: self currentEvent! !

!SystemWindow methodsFor: 'menu' stamp: 'sw 9/29/1999 10:45'!
sendToBack
	| aWorld nextWindow |
	aWorld _ self world.
	nextWindow _ aWorld submorphs detect:
		[:m | (m isKindOf: SystemWindow) and:  [m ~~ self]] ifNone: [^ self].
	nextWindow activate.
	aWorld addMorphNearBack: self
! !

!SystemWindow methodsFor: 'object fileIn' stamp: 'sw 9/28/1999 14:30'!
convertbosfcebbmsolslccappcfim0: varDict bosfcebbmsolslccappcfimm0: smartRefStrm
	"These variables are automatically stored into the new instance ('labelString' 'stripes' 'label' 'closeBox' 'collapseBox' 'activeOnlyOnTop' 'paneMorphs' 'paneRects' 'collapsedFrame' 'fullFrame' 'isCollapsed' 'menuBox' ).
	This method is for additional changes. Use statements like (foo _ varDict at: 'foo')."

	"New variables: ('mustNotClose' )  If a non-nil value is needed, please assign it."
	mustNotClose _ false
! !


!FlashPlayerWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 10:14'!
adjustBookControls
	| inner |
	startButton ifNil: [^ self].
	startButton align: startButton topLeft with: (inner _ self innerBounds) topLeft + (35@-4).
	progress ifNotNil:
		[progress align: progress topLeft with: (startButton right @ inner top) + (10@0)].
	stopButton align: stopButton topRight with: inner topRight - (16@4)! !


!PartsWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:51'!
adjustBookControls
	| inner |
	prevButton ifNil: [^ self].
	prevButton align: prevButton topLeft with: (inner _ self innerBounds) topLeft + (32 @ -1).
	nextButton align: nextButton topRight with: inner topRight - (18 @ 1).
	menuButton align: menuButton topLeft with: inner topRight + (-42 @ 5).! !

!PartsWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:40'!
initialize
	| aFont aForm |
	super initialize.
	color _ Color white.
	openForEditing _ false.
	aFont _ Preferences fontForScriptorButtons.
	self addMorph: (prevButton _ SimpleButtonMorph new borderWidth: 0;
			label: '<' font: aFont; color: Color transparent;
			setBalloonText: 'previous page';
			actionSelector: #previousPage; target: self; extent: 16@16).
	self addMorph: (nextButton _ SimpleButtonMorph new borderWidth: 0;
			label: '>' font: aFont; color: Color transparent;
			setBalloonText: 'next page';
			actionSelector: #nextPage; target: self; extent: 16@16).
	menuButton _ ThreePhaseButtonMorph new onImage: (aForm _ ScriptingSystem formAtKey: 'OfferToUnlock'); offImage: (ScriptingSystem formAtKey: 'OfferToLock'); pressedImage: (ScriptingSystem formAtKey: 'OfferToLock'); extent: aForm extent; state: #on.
	menuButton target: self; actionSelector: #toggleStatus; actWhen: #buttonUp.
	menuButton setBalloonText: 'open for editing'.
	self addMorph: menuButton.
"
	self addMorph: (menuButton _ SimpleButtonMorph new borderWidth: 0;
			label: '' font: aFont; color: Color transparent;
			actWhen: #buttonDown;
			actionSelector: #invokePartsWindowMenu; target: self; extent: 16@16)."

	self adjustBookControls! !

!PartsWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:39'!
setLabelWidgetAllowance
	^ labelWidgetAllowance _ 115! !


!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:37'!
adjustBookControls
	| inner |
	proceedButton ifNil: [^ self].
	proceedButton align: proceedButton topLeft with: (inner _ self innerBounds) topLeft + (35@-4).
	debugButton align: debugButton topRight with: inner topRight - (16@4).! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:37'!
initialize
	| aFont proceedLabel debugLabel aWidth |
	super initialize.
	(aWidth _ self widthOfFullLabelText) > 280 ifTrue: [^ self].   "No proceed/debug buttons if title too long"
	"self removeMenuBox."
	aWidth > 210
		ifTrue: "Abbreviated buttons if title pretty long"
			[proceedLabel _ 'p'.
			debugLabel _ 'd']
		ifFalse: "Full buttons if title short enough"
			[proceedLabel _ 'proceed'.
			debugLabel _ 'debug'].
	aFont _ Preferences fontForScriptorButtons.
	self addMorph: (proceedButton _ SimpleButtonMorph new borderWidth: 0;
			label: proceedLabel font: aFont; color: Color transparent;
			actionSelector: #proceed; target: self).
	proceedButton setBalloonText: 'continue execution'.
	self addMorph: (debugButton _ SimpleButtonMorph new borderWidth: 0;
			label: debugLabel font: aFont; color: Color transparent;
			actionSelector: #debug; target: self).
	debugButton setBalloonText: 'bring up a debugger'.

	proceedButton submorphs first color: Color blue.
	debugButton submorphs first color: Color red.

	self adjustBookControls! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:38'!
setLabelWidgetAllowance
	^ labelWidgetAllowance _ 180! !


!SystemWindowWithButton methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:31'!
adjustExtraButton
	buttonInTitle ifNil: [^ self].
	buttonInTitle align: buttonInTitle topLeft with:  self innerBounds topRight - (buttonInTitle width + 15 @ 1)! !

!SystemWindowWithButton methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:26'!
extent: newExtent
	super extent: (newExtent max: 120 @ 50).
	self adjustExtraButton! !

!SystemWindowWithButton methodsFor: 'as yet unclassified' stamp: 'sw 9/29/1999 07:27'!
setLabelWidgetAllowance
	^ labelWidgetAllowance _ 115! !


SystemWindow removeSelector: #goBehind!
ObjectExplorerWindow removeSelector: #relabel!
SystemWindowWithButton removeSelector: #initialize!
"Postscript:"
ScriptingSystem installSolidMenuForm.
ScriptingSystem saveForm: ((ScriptingSystem formAtKey: 'SolidMenu') magnifyBy: 0.5) atKey: 'TinyMenu'.
Preferences enable: #allowLabelDragging.
Utilities replaceToolsFlap.!

