'From Squeak 2.1 of June 30, 1998 on 10 September 1998 at 4:17:30 pm'!
"Change Set:		versionsChanges-sw
Date:			10 September 1998
Author:			Scott Wallace

Makes ChangeList scanVersionsOf:... no longer put up an informer before returning nil in case of an unlogged method; senders of this method put now up the informer when they want to (at least two senders do not). This change was motivated by the appearance of an unwanted informer when putting up the menu title for the selector menu for an unlogged method, and in any case the current approach cleaner and more flexible.

Also, herewith is added the ability for an author to hard-code the name to be used in the changeset-preamble-template, by reimplementing Preferences.defaultAuthorName"!


!Behavior methodsFor: 'accessing method dictionary' stamp: 'sw 9/10/1998 14:37'!
changeRecordsAt: selector
	"Return a list of ChangeRecords for all versions of the method at selector. Source code can be retrieved by sending string to any one.  Return nil if the method is absent."

	"(Pen changeRecordsAt: #go:) collect: [:cRec | cRec string]"
	| aList |
	aList _ ChangeList new
			scanVersionsOf: (self compiledMethodAt: selector ifAbsent: [^ nil])
			class: self meta: self isMeta
			category: (self whichCategoryIncludesSelector: selector)
			selector: selector.
	^ aList ifNotNil: [aList changeList]! !


!ChangeList methodsFor: 'scanning' stamp: 'sw 9/10/1998 15:17'!
scanVersionsOf: method class: class meta: meta category: category selector: selector
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp |
	changeList _ OrderedCollection new.
	list _ OrderedCollection new.
	listIndex _ 0.
	position _ method filePosition.
	sourceFilesCopy _ SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	method fileIndex == 0 ifTrue: [^ nil].
	file _ sourceFilesCopy at: method fileIndex.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  "Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [preamble _ file nextChunk].

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos _ nil.
		stamp _ ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens _ Scanner new scanTokens: preamble]
			ifFalse: [tokens _ Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp _ tokens at: tokens size-2.
						prevPos _ tokens last.
						prevFileIndex _ prevPos // 16r1000000.
						prevPos _ prevPos \\ 16r1000000]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos _ tokens at: tokens size-2.
						prevFileIndex _ tokens last].
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos _ nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp _ tokens at: tokens size]].
 		self addItem:
				(ChangeRecord new file: file position: position type: #method
						class: class name category: category meta: meta stamp: stamp)
			text: stamp , ' ' , class name , (meta ifTrue: [' class '] ifFalse: [' ']) , selector.
		position _ prevPos.
		prevPos notNil ifTrue:
			[file _ sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections _ Array new: list size withAll: false! !


!ChangeList class methodsFor: 'public access' stamp: 'sw 9/10/1998 15:17'!
browseVersionsOf: method class: class meta: meta
		category: category selector: selector 
	| changeList |
	Cursor read showWhile:
		[changeList _ self new
			scanVersionsOf: method class: class meta: meta
			category: category selector: selector].
	changeList
		ifNil:
			[self inform: 'No versions available']
		ifNotNil:
			[self open: changeList name: 'Recent versions of ' , selector multiSelect: false]! !

!ChangeList class methodsFor: 'public access' stamp: 'sw 9/10/1998 15:18'!
browseVersionsOf: method class: class meta: meta
		category: category selector: selector lostMethodPointer: sourcePointer
	| changeList |
	Cursor read showWhile:
		[changeList _ self new
			scanVersionsOf: method class: class meta: meta
			category: category selector: selector].
	changeList
		ifNil:
			[self inform: 'No versions available']
		ifNotNil:
			[changeList setLostMethodPointer: sourcePointer.
			self open: changeList name: 'Recent versions of ' , selector multiSelect: false]! !

!ChangeList class methodsFor: 'public access' stamp: 'sw 9/10/1998 15:39'!
versionCountForSelector: aSelector class: aClass
	"Answer the number of versions known to the system for the given class and method, including the current version.  A result of greater than one means that there is at least one superseded version.  6/28/96 sw"
	
	| method aChangeList |
	method _ aClass compiledMethodAt: aSelector.
	aChangeList _ self new
			scanVersionsOf: method class: aClass meta: aClass isMeta
			category: nil selector: aSelector.
	^ aChangeList ifNil: [0] ifNotNil: [aChangeList list size]! !


!ChangeSet methodsFor: 'fileIn/Out' stamp: 'sw 9/10/1998 12:37'!
preambleTemplate
	"Answer a string that will form the default contents for a change set's preamble.
	Just a first stab at what the content should be."

	^ String streamContents: [:strm |
		strm nextPutAll: '"Change Set:'.  "NOTE: fileIn recognizes preambles by this string."
		strm tab;tab; nextPutAll: self name.
		strm cr; nextPutAll: 'Date:'; tab; tab; tab; nextPutAll: Date today printString.
		strm cr; nextPutAll: 'Author:'; tab; tab; tab; nextPutAll: Preferences defaultAuthorName.
		strm cr; cr; nextPutAll: '<your descriptive text goes here>"']
"Smalltalk changes preambleTemplate"! !


!Preferences class methodsFor: 'hard-coded preferences' stamp: 'sw 9/10/1998 16:09'!
defaultAuthorName
	"Answer the author name to be planted, by default, in a changeset-preamble template.  You can hard-code this to hold your name, thus saving you time when writing the preambles of subsequent changesets"

	^ 'Your Name'! !


