'From Squeak2.8 of 2 August 2000 [latest update: #2348] on 11 August 2000 at 5:34:03 pm'!
"Change Set:		SetMacTypeForSaveAsJMM
Date:			10 August 2000
Author:			John M McIntosh

As part of making Squeak more macintosh friendly it should set the file type and owner correctly when you do an image save as, condense changes, or externalize sources after internalizing"!


!SystemDictionary methodsFor: 'sources, change log' stamp: 'JMM 8/10/2000 15:15'!
externalizeSources   
	"Write the sources and changes streams onto external files."
 	"Smalltalk externalizeSources"

	| sourcesName changesName aFile |
	sourcesName _ self sourcesName.
	(FileDirectory default fileExists: sourcesName)
		ifTrue: [^ self inform:
'Sorry, you must first move or remove the
file named ', sourcesName].
	changesName _ self changesName.
	(FileDirectory default fileExists: changesName)
		ifTrue: [^ self inform:
'Sorry, you must first move or remove the
file named ', changesName].

	aFile _  FileStream newFileNamed: sourcesName.
	aFile nextPutAll: SourceFiles first originalContents.
	aFile close.
	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: sourcesName
		type: 'STch'
		creator: 'FAST'.
	SourceFiles at: 1 put: (FileStream readOnlyFileNamed: sourcesName).

	aFile _ FileStream newFileNamed: self changesName.
	aFile nextPutAll: SourceFiles last contents.
	aFile close.
	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: self changesName
		type: 'STch'
		creator: 'FAST'.
	SourceFiles at: 2 put: (FileStream oldFileNamed: changesName).

	self inform: 'Sources successfully externalized'.
! !

!SystemDictionary methodsFor: 'snapshot and quit' stamp: 'JMM 8/10/2000 15:09'!
saveAs

	| dir newName newImageName newChangesName newImageSegDir oldImageSegDir haveSegs |
	dir _ FileDirectory default.
	newName _ FillInTheBlank
		request: 'New File Name?'
		initialAnswer: (FileDirectory localNameFor: self imageName).
	newName = '' ifTrue: [^self].
	newName _ FileDirectory baseNameFor: newName asFileName.

	newImageName _ newName, FileDirectory dot, FileDirectory imageSuffix.
	newChangesName _ newName, FileDirectory dot, FileDirectory changeSuffix.
	((dir includesKey: newImageName) or:
	 [dir includesKey: newChangesName]) ifTrue: [
		^ self notify: newName, ' is already in use.
Please choose another name.'].

	dir copyFileNamed: self changesName toFileNamed: newChangesName.
	
	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: newChangesName
		type: 'STch'
		creator: 'FAST'.

	haveSegs _ false.
	Smalltalk at: #ImageSegment ifPresent: [:theClass | 
		(haveSegs _ theClass instanceCount ~= 0) ifTrue: [
			oldImageSegDir _ theClass segmentDirectory]].

	self logChange: '----SAVEAS ', newName, '----', Date dateAndTimeNow printString.
	self imageName: (dir fullNameFor: newImageName).
	LastImageName _ self imageName.
	self closeSourceFiles; openSourceFiles.  "so SNAPSHOT appears in new changes file"
	haveSegs ifTrue: [
		Smalltalk at: #ImageSegment ifPresent: [:theClass |
			newImageSegDir _ theClass segmentDirectory.	"create the folder"
			oldImageSegDir fileNames do: [:theName | "copy all segment files"
				newImageSegDir 
					copyFileNamed: oldImageSegDir pathName, FileDirectory slash, theName 
					toFileNamed: theName]]].
	self snapshot: true andQuit: false.
! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'JMM 8/10/2000 15:13'!
condenseSources		"Smalltalk condenseSources"
	"Move all the changes onto a compacted sources file."
	| f classCount dir |
	dir _ FileDirectory default.

	"Write all sources with fileIndex 1"
	f _ FileStream newFileNamed: self sourcesName , '.temp'.
	f header; timeStamp.
'Condensing Sources File...'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: Smalltalk classNames size
	during:
		[:bar | classCount _ 0.
		Smalltalk allClassesDo:
			[:class | bar value: (classCount _ classCount + 1).
			class fileOutOn: f moveSource: true toFile: 1]].
	f trailer; close.

	"Make a new empty changes file"
	self closeSourceFiles.
	dir rename: self changesName
		toBe: self changesName , '.old'.
	(FileStream newFileNamed: self changesName)
		header; timeStamp; close.
	LastQuitLogPosition _ 0.

	dir rename: self sourcesName
		toBe: self sourcesName , '.old'.
	dir rename: self sourcesName , '.temp'
		toBe: self sourcesName.

	"On Mac, set the file type and creator (noop on other platforms)"
	FileDirectory default
		setMacFileNamed: self changesName
		type: 'STch'
		creator: 'FAST'.

	FileDirectory default
		setMacFileNamed:  self sourcesName
		type: 'STch'
		creator: 'FAST'.

	self openSourceFiles.
	SelectionMenu notify: 'Source files have been rewritten!!
Check that all is well,
and then save/quit.'! !

