'From Squeak2.9alpha of 5 August 2000 [latest update: #3220] on 4 January 2001 at 12:42:39 pm'!
"Change Set:		miscTweaks-sw
Date:			4 January 2001
Author:			Scott Wallace

Changes 'revert and forget' wording to the more accurate'revert & remove from changes'.
Removes method SequenceableCollection.asDictionary, an ancient and long disused method that recently raised complaints on the Squeak list.
Adds another annotation possibility, #firstComment, which will extract the first comment from the method, if any, and place it in the annotation pane."!


!CodeHolder methodsFor: 'annotation' stamp: 'sw 1/3/2001 18:22'!
annotation
	"Provide a line of annotation material for a middle pane."

	| stamp  sendersCount implementorsCount toShow aCategory separator aString aList versionsCount aClass aSelector aComment |

	((aSelector _ self selectedMessageName) == nil or: [(aClass _ self selectedClassOrMetaClass) == nil])
		ifTrue: [^ '------'].
	
	toShow _ ReadWriteStream on: ''.
	separator _ '  '.
	self annotationRequests do:
		[:aRequest |
		(aRequest == #firstComment) ifTrue:
			[aComment _ aClass firstCommentAt:  aSelector.
			aComment isEmptyOrNil ifFalse:
				[toShow nextPutAll: (aComment, separator)]].

		(aRequest == #timeStamp) ifTrue:
			[stamp _ self timeStamp.
			toShow nextPutAll: (stamp size > 0
				ifTrue: [stamp, separator]
				ifFalse: ['no timeStamp', separator])].

		(aRequest == #messageCategory) ifTrue:
			[aCategory _ aClass organization categoryOfElement: aSelector.
			aCategory ifNotNil: "woud be nil for a method no longer present, e.g. in a recent-submissions browser"
				[toShow nextPutAll: aCategory, separator]].

		(aRequest == #sendersCount) ifTrue:
			[sendersCount _ (Smalltalk allCallsOn: aSelector) size.
			sendersCount _ sendersCount == 1
				ifTrue:
					['1 sender']
				ifFalse:
					[sendersCount printString, ' senders'].
			toShow nextPutAll: sendersCount, separator].

		(aRequest == #implementorsCount) ifTrue:
			[implementorsCount _ (Smalltalk allImplementorsOf: aSelector) size.
			implementorsCount _ implementorsCount == 1
				ifTrue:
					['1 implementor']
				ifFalse:
					[implementorsCount printString, ' implementors'].
			toShow nextPutAll: implementorsCount,  separator].

		(aRequest == #priorVersionsCount) ifTrue:
			[versionsCount _ VersionsBrowser versionCountForSelector: aSelector class: aClass.
			toShow nextPutAll: 
				((versionsCount > 1
					ifTrue:
						[versionsCount == 2 ifTrue:
							['1 prior version']
							ifFalse:
								[versionsCount printString, ' prior versions']]
					ifFalse:
						['no prior versions']), separator)].

		(aRequest == #priorTimeStamp) ifTrue:
			[stamp _ VersionsBrowser timeStampFor: aSelector class: aClass reverseOrdinal: 2.
			stamp ifNotNil: [toShow nextPutAll: 'prior time stamp: ', stamp, separator]].

		(aRequest == #recentChangeSet) ifTrue:
			[aString _ ChangeSorter mostRecentChangeSetWithChangeForClass: aClass selector: aSelector.
			aString size > 0 ifTrue: [toShow nextPutAll: aString, separator]].

		(aRequest == #allChangeSets) ifTrue:
			[aList _ ChangeSorter allChangeSetsWithClass: aClass selector: aSelector.
			aList size > 0
				ifTrue:
					[aList size = 1
						ifTrue:
							[toShow nextPutAll: 'only in change set ']
						ifFalse:
							[toShow nextPutAll: 'in change sets: '].
					aList do:
						[:aChangeSet | toShow nextPutAll: aChangeSet name, ' ']]
				ifFalse:
					[toShow nextPutAll: 'in no change set'].
			toShow nextPutAll: separator]].
		
	^ toShow contents! !


!ChangeSorter methodsFor: 'message list' stamp: 'sw 1/4/2001 10:22'!
shiftedMessageMenu: aMenu
	"Arm the menu so that it holds items appropriate to the message-list while the shift key is down.  Answer the menu."

	^ aMenu addList: #(
		-
		('method pane'						makeIsolatedCodePane)
		('toggle diffing'						toggleDiffing)
		('implementors of sent messages'		browseAllMessages)
		('change category...'				changeCategory)
			-
		('sample instance'					makeSampleInstance)
		('inspect instances'					inspectInstances)
		('inspect subinstances'				inspectSubInstances)
		-
		('change sets with this method'		findMethodInChangeSets)
		('revert to previous version'			revertToPreviousVersion)
		('revert & remove from changes'	revertAndForget)
		-
		('more...'							unshiftedYellowButtonActivity))! !


!Debugger methodsFor: 'context stack menu' stamp: 'sw 1/4/2001 10:23'!
contextStackMenu: aMenu shifted: shifted
	"Set up the menu appropriately for the context-stack-list, either shifted or unshifted as per the parameter provided"

	^ shifted ifFalse: [aMenu labels: 
'fullStack (f)
restart (r)
proceed (p)
step (t)
send (e)
where (w)
peel to first like this
senders of... (n)
implementors of... (m)
inheritance (i)
versions (v)
inst var refs...
inst var defs...
class var refs...
class variables
class refs (N)
browse full (b)
file out 
mail out bug report
more...'
	lines: #(7 11 13 16 19)
	selections: #(fullStack restart proceed doStep send where peelToFirst
browseSendersOfMessages browseMessages methodHierarchy browseVersions
browseInstVarRefs browseInstVarDefs
browseClassVarRefs browseClassVariables browseClassRefs
browseMethodFull fileOutMessage mailOutBugReport
shiftedYellowButtonActivity)]

	ifTrue: [aMenu labels: 
'browse class hierarchy
browse class
browse method (O)
implementors of sent messages
change sets with this method
inspect instances
inspect subinstances
revert to previous version
remove from current change set
revert & remove from changes
more...' 
	lines: #(5 7 10)
	selections: #(classHierarchy browseClass 
		openSingleMessageBrowser browseAllMessages findMethodInChangeSets 
		inspectInstances inspectSubInstances
		revertToPreviousVersion 
		removeFromCurrentChanges revertAndForget
		unshiftedYellowButtonActivity)]

! !


!Preferences class methodsFor: 'parameters' stamp: 'sw 1/4/2001 10:25'!
annotationInfo 
	"Answer a list of pairs characterizing all the available kinds of annotations; in each pair, the first element is a symbol representing the info type, and the second element is a string providing the corresponding balloon help"

	^ #(

		(timeStamp			'The time stamp of the last submission of the method.')
		(firstComment		'The first comment in the method, if any.')
		(messageCategory	'Which method category the method lies in')
		(sendersCount		'A report of how many senders there of the message.')
		(implementorsCount	'A report of how many implementors there are of the message.')
		(recentChangeSet	'The most recent change set bearing the method.')
		(allChangeSets		'A list of all change sets bearing the method.')
		(priorVersionsCount	'A report of how many previous versions there are of the method' )
		(priorTimeStamp		'The time stamp of the penultimate submission of the method, if any'))! !

SequenceableCollection removeSelector: #asDictionary!