'From Squeak2.9alpha of 17 July 2000 [latest update: #3245] on 10 January 2001 at 10:18:21 am'!
"Change Set:		killDualResizers
Date:			10 January 2001
Author:			Bob Arning

Fix the situation where there were sometimes two yellow resizer handles in a SystemWindow."!


!NewHandleMorph methodsFor: 'all' stamp: 'RAA 1/10/2001 10:15'!
morphicLayerNumber

	^1		"handles are very front-like - e.g. the spawn reframe logic actually asks if the first submorph of the world is one of us before deciding to create one"! !


!SystemWindow methodsFor: 'resize/collapse' stamp: 'RAA 1/10/2001 10:10'!
spawnPaneFrameHandle: event
	| resizer localPt side growingPane newBounds adjoiningPanes limit |
	(self world firstSubmorph isKindOf: NewHandleMorph) ifTrue:
		[ ^ self  "Prevent multiple handles"].
	((self innerBounds withHeight: self labelHeight+4) containsPoint: event cursorPoint)
		ifTrue: [^ self "in label or top of top pane"].
	growingPane _ self paneWithLongestSide: [:s | side _ s] near: event cursorPoint.
	growingPane ifNil: [^ self].
	"don't resize pane side coincident with window side - RAA 5 jul 2000"
	(growingPane perform: side) = (self innerBounds perform: side) ifTrue: [^self].
	(side == #top and: [growingPane top = self panelRect top]) ifTrue: [^self].
	adjoiningPanes _ paneMorphs select: [:pane | pane bounds bordersOn: growingPane bounds along: side].
	limit _ adjoiningPanes isEmpty
		ifFalse: [ (adjoiningPanes collect: [:pane | pane bounds perform: side])
			perform: ((#(top left) includes: side) ifTrue: [#max] ifFalse: [#min])]
		ifTrue: [self bounds perform: side].

	resizer _ NewHandleMorph new
		followHand: event hand
		forEachPointDo:
			[:p | localPt _ self pointFromWorld: p.
			newBounds _ growingPane bounds withSideOrCorner: side setToPoint: localPt minExtent: 40@20 limit: limit.
			self fastFramingOn 
			ifTrue:
				["For fast display, only higlight the rectangle during loop"
				newBounds _ growingPane bounds
				  newRectButtonPressedDo:
					[:f | growingPane bounds withSideOrCorner: side
							setToPoint: (self pointFromWorld: Sensor cursorPoint)
							minExtent: 40@20 limit: limit].
					self reframePanesAdjoining: growingPane along: side
						to: newBounds]
			ifFalse:
				[self reframePanesAdjoining: growingPane along: side to: newBounds]]
		lastPointDo: [:p | ].
	event hand world addMorphInLayer: resizer.
	resizer startStepping! !

!SystemWindow methodsFor: 'resize/collapse' stamp: 'RAA 1/10/2001 10:12'!
spawnReframeHandle: event
	"The mouse has crossed a pane border.  Spawn a reframe handle."
	| resizer localPt pt ptName newBounds |
	allowReframeHandles ifFalse: [^ self].
	owner ifNil: [^ self  "Spurious mouseLeave due to delete"].
	(self isActive not or: [self isCollapsed]) ifTrue:  [^ self].
	((self world ifNil: [^ self]) firstSubmorph isKindOf: NewHandleMorph) ifTrue:
		[^ self  "Prevent multiple handles"].
	pt _ event cursorPoint.
	self bounds forPoint: pt closestSideDistLen:
		[:side :dist :len |  "Check for window side adjust"
		dist <= 2  ifTrue: [ptName _ side]].
	ptName ifNil:
		["Check for pane border adjust"
		^ self spawnPaneFrameHandle: event].
	#(topLeft bottomRight bottomLeft topRight) do:
		[:corner |  "Check for window corner adjust"
		(pt dist: (self bounds perform: corner)) < 20 ifTrue: [ptName _ corner]].

	resizer _ NewHandleMorph new
		followHand: event hand
		forEachPointDo:
			[:p | localPt _ self pointFromWorld: p.
			newBounds _ self bounds
				withSideOrCorner: ptName
				setToPoint: localPt
				minExtent: self minimumExtent.
			self fastFramingOn 
			ifTrue:
				[self doFastWindowReframe: ptName]
			ifFalse:
				[self bounds: newBounds.
				(Preferences roundedWindowCorners
					and: [#(bottom right bottomRight) includes: ptName])
					ifTrue:
					["Complete kluge: causes rounded corners to get painted correctly,
					in spite of not working with top-down displayWorld."
					ptName = #bottom ifFalse:
						[self invalidRect: (self bounds topRight - (6@0) extent: 7@7)].
					ptName = #right ifFalse:
						[self invalidRect: (self bounds bottomLeft - (0@6) extent: 7@7)].
					self invalidRect: (self bounds bottomRight - (6@6) extent: 7@7)]]]
		lastPointDo:
			[:p | ].
	event hand world addMorphInLayer: resizer.
	resizer startStepping! !

