'From Squeak3.1alpha of 28 February 2001 [latest update: #4337] on 14 September 2001 at 00:00:01 am'!
"Change Set:		vocabRefactor-sw
Date:			14 September 2001
Author:			Scott Wallace

Refactors Vocabulary architecture, making the choice of natural language an independent dimension, with actual translations instituted lazily only when required by user request.

Every project has a natural language associated with it; default is English.  When you switch natural languages (there is a new item in the Help menu to do this,) the impact will only be seriously felt in the classic tile-scripting regime, where all viewers and scriptors will immediately be re-rendered in the new language.  Ultimately, broader localization would be desirable -- menus, balloon help, labels, and much else.

Refactors ObjectWithDocumentation to provide direct support for multiple natural languages.

Attention translators: consult EToyVocabulary method #templateForLanguageTranslation -- this method is provided to assist people wishing to translate the etoy experience into another natural language.  If you carefully follow the instructions in the comment at the front of that method, it should be straightforward to translate to any language.

Note to Alan and Ted:  This refactoring recasts the 'players are vectors' thing as a separate vocabulary.  To switch any particular *viewer* over to using the Vector vocabulary, use 'choose vocabulary' from the viewer's menu; to switch an entire *project* over to the 'players are vectors' mode, evaluate 'ActiveWorld installVectorVocabulary.', which will be found at the bottom of the 'do...' menu.  To rescind a project-wide vocabulary choice, evaluate 'ActiveWorld abandonVocabularyPreference.', also found in the 'do...' menu."!

Object subclass: #ElementTranslation
	instanceVariableNames: 'wording helpMessage naturalLanguageSymbol '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Protocols'!

Object subclass: #ObjectWithDocumentation
	instanceVariableNames: 'authoringStamp documentation properties elementSymbol naturalLanguageTranslations '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Protocols'!

ObjectWithDocumentation subclass: #Vocabulary
	instanceVariableNames: 'vocabularyName categories methodInterfaces object limitClass translationTable '
	classVariableNames: 'AllVocabularies AllMethodInterfaces AllStandardVocabularies  '
	poolDictionaries: ''
	category: 'System-Protocols'!


EToyVocabulary subclass: #EToyVectorVocabulary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Protocols'!

!EToyVectorVocabulary commentStamp: '<historical>' prior: 0!
An extension of the etoy vocabulary in support of an experiment Alan Kay requested in summer 2001 for allowing any morph/player to be thought of as a vector.  In effect, adds a category #vector to the viewer for such all morphs.  Consult Ted Kaehler and Alan Kay for more information on this track.!


!ElementTranslation methodsFor: 'initialization' stamp: 'sw 9/12/2001 23:00'!
language
	"Answer the natural language symbol of the receiver"

	^ naturalLanguageSymbol ifNil: [naturalLanguageSymbol _ #English]! !

!ElementTranslation methodsFor: 'initialization' stamp: 'sw 9/12/2001 10:15'!
wording: aWording helpMessage: aHelpMessage language: aLanguageSymbol
	"Set state directly"

	wording _ aWording.
	helpMessage _ aHelpMessage.
	naturalLanguageSymbol _ aLanguageSymbol! !

!ElementTranslation methodsFor: 'printing' stamp: 'sw 9/12/2001 16:08'!
printOn: aStream
	"Print the receiver on the stream"

	aStream nextPutAll: '('.
	super printOn: aStream.
	aStream nextPutAll: ' language: ', naturalLanguageSymbol asString, ' wording: ', wording asString, ')'! !


!ElementTranslation class methodsFor: 'instance creation' stamp: 'sw 9/12/2001 10:16'!
fromPair: wordingAndHelpMessagePair language: aLanguageSymbol
	"Answer an instance with the given wording and help message"

	^ self new wording: wordingAndHelpMessagePair first helpMessage: wordingAndHelpMessagePair second language: aLanguageSymbol! !


!Lexicon methodsFor: 'vocabulary' stamp: 'sw 9/6/2001 15:06'!
chooseVocabulary
	"Put up a dialog affording the user a chance to choose a different vocabulary to be installed in the receiver"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Choose a vocabulary
blue = current
red = imperfect'.
	aMenu addStayUpItem.
	Vocabulary allStandardVocabularies do:
		[:aVocabulary |
			(targetClass implementsVocabulary: aVocabulary)
				ifTrue:
					[aMenu add: aVocabulary vocabularyName selector: #switchToVocabulary: argument: aVocabulary.
					(targetClass fullyImplementsVocabulary: aVocabulary) ifFalse:
						[aMenu lastItem color: Color red].
					aVocabulary == currentVocabulary ifTrue:
						[aMenu lastItem color: Color blue]. 
					aMenu balloonTextForLastItem: aVocabulary documentation]].
	aMenu popUpInWorld: self currentWorld! !


!Morph methodsFor: 'e-toy support' stamp: 'sw 9/12/2001 20:03'!
setNaturalLanguageTo: aLanguageSymbol
	"Set the natural language of this morph to be as indicated"! !


!Morph class methodsFor: 'scripting' stamp: 'sw 9/10/2001 08:36'!
additionsToViewerCategory: aCategoryName
	"Answer a list of viewer specs for items to be added to the given category on behalf of the receiver.  Each class in a morph's superclass chain is given the opportunity to add more things"

	aCategoryName == #vector ifTrue:
		[^ self vectorAdditions].
	self additionsToViewerCategories do:
		[:anAddition | anAddition first == aCategoryName ifTrue: [^ anAddition second]].
	^ #()! !

!Morph class methodsFor: 'scripting' stamp: 'sw 9/12/2001 23:00'!
additionsToViewerCategoryGeometry
	"answer additions to the geometry viewer category"

	^ #(geometry 
		(
			(slot  scaleFactor 'The factor by which the object is magnified' number readWrite player getScaleFactor player setScaleFactor:)
			(slot  left   'The left edge' number readWrite player getLeft  player  setLeft:)
			(slot right  'The right edge' number readWrite player getRight  player  setRight:)
			(slot  top  'The top edge' number readWrite player getTop  player  setTop:) 
			(slot  bottom  'The bottom edge' number readWrite player getBottom  player  setBottom:) 
			(slot  width  'The width' number readWrite player getWidth  player  setWidth:)
			(slot  height  'The height' number readWrite player getHeight  player  setHeight:) 
			(slot x   'The x coordinate' number readWrite player  getX   player setX:)
			(slot y   'The y coordinate' number readWrite player  getY  player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' number readWrite player getHeading  player setHeading:)
			(slot distance 'The length of the vector connecting the origin to the object''s position' number readWrite player getDistance player setDistance:)
			(slot theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position' number readWrite player getTheta player setTheta: )
		)
	)


! !

!Morph class methodsFor: 'scripting' stamp: 'sw 9/12/2001 23:01'!
vectorAdditions
	"Answer slot/command definitions for the vector experiment"

	^ # (
(slot x   'The x coordinate' number readWrite player  getX   player setX:)
(slot y   'The y coordinate' number readWrite player  getY  player setY:)
(slot heading  'Which direction the object is facing.  0 is straight up' number readWrite player getHeading  player setHeading:)
(slot distance 'The length of the vector connecting the origin to the object''s position' number readWrite player getDistance player setDistance:)
(slot theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position' number readWrite player getTheta player setTheta: )

(command + 'Adds two players together, treating each as a vector from the origin.' player)
(command - 'Subtracts one player from another, treating each as a vector from the origin.' player)
(command * 'Multiply a player by a number, treating the Player as a vector from the origin.' number)
(command / 'Divide a player by a number, treating the Player as a vector from the origin.' number)

(command incr: 'Each Player is a vector from the origin.  Increase one by the amount of the other.' player)
(command decr: 'Each Player is a vector from the origin.  Decrease one by the amount of the other.' player)
(command multBy: 'A Player is a vector from the origin.  Multiply its length by the factor.' number)
(command dividedBy: 'A Player is a vector from the origin.  Divide its length by the factor.' number)
	)! !


!ObjectWithDocumentation methodsFor: 'documentation' stamp: 'sw 9/12/2001 14:47'!
documentation: maneno
	"Set the receiver's documentation, in the current langauge"

	| wording |
	wording _ self wording.
	self absorbTranslation: (ElementTranslation new wording: wording  helpMessage: maneno third language: #English) ! !

!ObjectWithDocumentation methodsFor: 'documentation' stamp: 'sw 9/12/2001 23:19'!
documentationOrNil
	"Answer the contents of the receiver's documentation slot.  This interface has perhaps outlived its usefulness."

	^ self documentation! !

!ObjectWithDocumentation methodsFor: 'queries' stamp: 'sw 9/12/2001 14:36'!
documentation
	"Answer the receiver's documentation"

	^ self translatedToPrevailingLanguage helpMessage! !

!ObjectWithDocumentation methodsFor: 'queries' stamp: 'sw 9/12/2001 15:13'!
wording
	"Answer the receiver's wording"

	^ self translatedToPrevailingLanguage wording! !

!ObjectWithDocumentation methodsFor: 'translation' stamp: 'sw 9/12/2001 23:03'!
absorbTranslation: anElementTranslation
	"Absorb the given translation, which could overlay a prior version"

	naturalLanguageTranslations ifNil:
		[naturalLanguageTranslations _ OrderedCollection new].
	naturalLanguageTranslations removeAllSuchThat:
		[:tr | tr language == anElementTranslation language].
	naturalLanguageTranslations add: anElementTranslation! !

!ObjectWithDocumentation methodsFor: 'translation' stamp: 'sw 9/12/2001 22:43'!
translatedToPrevailingLanguage
	"Answer an ElementTranslation object in the prevailing natural language, or, if none found, in English"

	| prevailingLanguage fallback elSym el wrd doc |

	prevailingLanguage _ ActiveWorld currentNaturalLanguage.
	elSym _ self elementSymbol.
	^ naturalLanguageTranslations isEmptyOrNil 
		ifFalse:
			[naturalLanguageTranslations do:
				[:aTranslation |
					aTranslation language == prevailingLanguage
						ifTrue:
							[^ aTranslation].
					aTranslation language == #English
						ifTrue:
							[fallback _ aTranslation]].
			fallback ifNil: [ElementCategory new categoryName: #none]]

		ifTrue:
			[(el _ self elementSymbol) ifNotNil:
				[((el beginsWith: 'get') and: [elSym size > 3])
					ifTrue:
						[wrd _ (elSym copyFrom: 4 to: elSym size) withFirstCharacterDownshifted.
						doc _ 'get value of ', elSym]
					ifFalse:
						[((elSym beginsWith: 'set') and: [elSym size > 4])
							ifTrue:
								[wrd _ (elSym copyFrom: 4 to: elSym size - 1) withFirstCharacterDownshifted.
								doc _ 'set value of ', elSym]
							ifFalse:
								[wrd _ ScriptingSystem wordingForOperator: elSym.
								doc _ nil]]].
			^ ElementTranslation new wording: wrd helpMessage: doc language: #English]! !

!ObjectWithDocumentation methodsFor: 'miscellaneous' stamp: 'sw 9/12/2001 23:03'!
elementSymbol
	"Answer the receiver's element symbol"

	^ elementSymbol! !


!ElementCategory methodsFor: 'elements' stamp: 'sw 9/12/2001 22:59'!
elementSymbol
	"Answer the element symbol for the receiver.  Here, the categoryName dominates"

	^ categoryName! !


!MethodInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 22:45'!
conjuredUpFor: aSelector class: aClass
	"Initialize the receiver to have the given selector, obtaining whatever info one can from aClass.  This basically covers the situation where no formal definition has been made."

	| parts |
	self initializeFor: aSelector.
	self absorbTranslation:
		(ElementTranslation new wording: aSelector helpMessage: 'no help available' language: #English).

	receiverType _ #unknown.
	parts _ aClass formalHeaderPartsFor: aSelector.
	argumentVariables _ (1 to: selector numArgs) collect:
		[:anIndex | Variable new name: (parts at: (4 * anIndex)) type: #object].
	parts last isEmptyOrNil ifFalse: [self documentation: parts last].
! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 12:38'!
initializeFromEToyCommandSpec: tuple category: aCategorySymbol
	"tuple holds an old etoy command-item spec, of the form found in #additionsToViewerCategories methods.   Initialize the receiver to hold the same information"

	selector _ tuple second.
	receiverType _ #player.
	selector numArgs == 1 ifTrue:
		[argumentVariables _ OrderedCollection with:
			(Variable new name: (Player formalHeaderPartsFor: selector) fourth type: tuple fourth)].

	aCategorySymbol ifNotNil: [self flagAttribute: aCategorySymbol].
	self absorbTranslation:
		(ElementTranslation new wording: (ScriptingSystem wordingForOperator: selector) helpMessage:  tuple third language: #English)! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 12:36'!
initializeFromEToySlotSpec: tuple
	"tuple holds an old etoy slot-item spec, of the form found in #additionsToViewerCategories methods.   Initialize the receiver to hold the same information"

	| setter |
	selector _ tuple seventh.
	self absorbTranslation:
		(ElementTranslation new wording: tuple second helpMessage: tuple third language: #English).

	receiverType _ #player.
	resultSpecification _ ResultSpecification new.
	resultSpecification resultType: tuple fourth.
	(#(getNewClone  "etc.") includes: selector)
		ifTrue:
			[self setNotToRefresh]  "actually should already be nil"
		ifFalse:
			[self setToRefetch].

	((tuple fifth == #readWrite) and: [((tuple size >= 9) and: [(setter _ tuple at: 9) ~~ #unused])]) ifTrue:
		[resultSpecification companionSetterSelector: setter].
		
"An example of an old slot-item spec:
(slot numericValue 'A number representing the current position of the knob.' number readWrite player getNumericValue player setNumericValue:)
	1	#slot
	2	wording
	3	balloon help
	4	type
	5	#readOnly or #readWrite
	6	#player (not used -- ignore)
	7	getter selector
	8	#player (not used -- ignore)
	9	setter selector
"
	! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 12:35'!
initializeSetterFromEToySlotSpec: tuple
	"tuple holds an old etoy slot-item spec, of the form found in #additionsToViewerCategories methods.   Initialize the receiver to represent the getter of this item"

	selector _ tuple ninth.
	self absorbTranslation:
		(ElementTranslation new wording: ('set ', tuple second) helpMessage: ('setter for', tuple third) language: #English).
	receiverType _ #player.
	argumentVariables _ Array with: (Variable new variableType: tuple fourth)
	! !

!MethodInterface methodsFor: 'access' stamp: 'sw 9/13/2001 16:42'!
elementSymbol
	"Answer the element symbol, for the purposes of translation"

	^ selector! !

!MethodInterface methodsFor: 'access' stamp: 'sw 9/12/2001 14:05'!
elementWording
	"Answer the wording to be shown on friendly tiles representing the receiver; by default, it is just the same as the method selector itself, but anything special-cased via #wordingForOperator:, and all getters/setters of slots, are transformed into somethingfriendlier here"

	^ self translatedToPrevailingLanguage wording! !


!MethodWithInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 11:59'!
playerClass: aPlayerClass selector: aSelector
	"Set the playerClass and selector of the receiver"

	playerClass _ aPlayerClass.
	selector _ aSelector.! !

!MethodWithInterface methodsFor: 'rename' stamp: 'sw 9/12/2001 11:59'!
renameScript: newSelector fromPlayer: aPlayer
	"The receiver's selector has changed to the new selector.  Get various things right, including the physical appearance of any Scriptor open on this method"

	self allScriptEditors do:
		[:aScriptEditor | aScriptEditor renameScriptTo: newSelector].
	selector _ newSelector asSymbol.
	self bringUpToDate.
	self playerClass atSelector: selector putScript: self.
! !


!PasteUpMorph methodsFor: 'world state' stamp: 'sw 9/13/2001 10:06'!
abandonVocabularyPreference
	"Remove any memory of a preferred vocabulary in the project"

	self removeProperty: #currentVocabularySymbol

"ActiveWorld abandonVocabularyPreference"! !

!PasteUpMorph methodsFor: 'world state' stamp: 'sw 9/12/2001 11:50'!
currentNaturalLanguage
	"Answer the natural language symbol of the current project's natural language"

	^ self project naturalLanguage

"ActiveWorld currentNaturalLanguage"! !

!PasteUpMorph methodsFor: 'world state' stamp: 'sw 9/13/2001 09:44'!
installVectorVocabulary
	"Install the experimental Vector vocabulary as the default for the current project"

	self setProperty: #currentVocabularySymbol toValue: #Vector! !


!Player methodsFor: 'slots-kernel' stamp: 'sw 9/13/2001 16:54'!
categoriesForVocabulary: aVocabulary
	"Answer a list of categories appropriate to the receiver and its costumes, in the given Vocabulary"

	| aList |
	self hasCostumeThatIsAWorld
		ifTrue:
			[aList _ self categoriesForWorld]
		ifFalse:
			[aList _ OrderedCollection new.
			self slotNames size > 0 ifTrue:
				[aList add: #'instance variables'].
			aList addAll: costume categoriesForViewer].
	aVocabulary addCustomCategoriesTo: aList.
	aList remove: #scripts ifAbsent: [].
	aList add: #scripts after: aList first.
	^ aList! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 9/12/2001 12:43'!
methodInterfacesForInstanceVariablesCategoryIn: aVocabulary
	"Return a collection of methodInterfaces for the instance-variables category.  The vocabulary parameter, at present anyway, is not used."

	| aList anInterface itsSlotName |
	aList _ OrderedCollection new.
	self slotInfo associationsDo:
		[:assoc |
			anInterface _ MethodInterface new.
			itsSlotName _ assoc key.
			anInterface absorbTranslation: 
				(ElementTranslation new wording: itsSlotName helpMessage: 'an instance variable defined by this object' language: #English).

			anInterface selector: (Utilities getterSelectorFor: itsSlotName) type: assoc value type setter: (Utilities setterSelectorFor: itsSlotName).
			anInterface setToRefetch.
			aList add: anInterface].
	^ aList! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/12/2001 12:42'!
methodInterfaceForEmptyScript
	"Answer a MethodInterface representing Andreas's 'emptyScript' feature"

	| anInterface |
	anInterface _ MethodInterface new.
	anInterface receiverType: #player.
	anInterface flagAttribute: #scripts.
	anInterface absorbTranslation: 
		(ElementTranslation new wording: (ScriptingSystem wordingForOperator: #emptyScript) helpMessage: 'an empty script; drop on desktop to get a new empty script for this object' language: #English).

	anInterface selector: #emptyScript type: nil setter: nil.
	^ anInterface! !


!Preferences class methodsFor: 'reacting to change' stamp: 'tk 7/18/2001 16:03'!
setNotificationParametersForStandardPreferences
	"Set up the notification parameters for the standard preferences that require need them.  When adding new Preferences that require use of the notification mechanism, users declare the notifcation info as part of the call that adds the preference, or afterwards -- the two relevant methods for doing that are:
 	Preferences.addPreference:categories:default:balloonHelp:projectLocal:changeInformee:changeSelector:   and
	Preference changeInformee:changeSelector:"

		"Preferences setNotificationParametersForStandardPreferences"

	| aPreference |
	#(	
		(annotationPanes		annotationPanesChanged)
		(eToyFriendly			eToyFriendlyChanged)
		(infiniteUndo			infiniteUndoChanged)
		(uniTilesClassic			classicTilesSettingToggled)
		(optionalButtons			optionalButtonsChanged)
		(roundedWindowCorners	roundedWindowCornersChanged)
		(showProjectNavigator	showProjectNavigatorChanged)
		(smartUpdating			smartUpdatingChanged)
		(universalTiles			universalTilesSettingToggled)
		(showSharedFlaps		sharedFlapsSettingChanged))  do:

			[:pair |
				aPreference _ self preferenceAt: pair first.
				aPreference changeInformee: self changeSelector: pair second]! !


!Project methodsFor: 'project parameters' stamp: 'sw 9/6/2001 12:04'!
projectParameterAt: aKey default: defaultValueBlock
	"Return the project parameter setting at the given key.  If there is no entry for this key in the Parameters dictionary, create one with the value of defaultValueBlock as its value"

	^ self projectParameters at: aKey ifAbsent: [self projectParameters at: aKey put: defaultValueBlock value]! !

!Project methodsFor: 'language' stamp: 'sw 9/12/2001 23:04'!
chooseNaturalLanguage
	"Put up a menu allowing the user to choose the natural language for the project"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'choose language'.
	aMenu lastItem setBalloonText: 'This controls the human language in which tiles should be viewed.  It is potentially extensible to be a true localization mechanism, but initially it only works in the classic tile scripting system.  Each project has its own private language choice'.
	aMenu addStayUpItem.
	#(English Deutsch Kiswahili) do:
		[:langSymbol |
			aMenu addUpdating: #stringForLanguageNameIs: target: self selector:  #setNaturalLanguageTo: argumentList: {langSymbol}].
	aMenu popUpInWorld

"Project current chooseNaturalLanguage"! !

!Project methodsFor: 'language' stamp: 'sw 9/12/2001 23:04'!
naturalLanguage
	"Answer the natural language for the project"

	^ self projectParameterAt: #naturalLanguage default: [#English]! !

!Project methodsFor: 'language' stamp: 'sw 9/13/2001 14:41'!
setNaturalLanguageTo: aLanguageSymbol
	"Set the project's natural language as indicated"

	Vocabulary assureTranslationsAvailableFor: aLanguageSymbol.
	aLanguageSymbol = self naturalLanguage ifFalse:
		[self projectParameterAt: #naturalLanguage put: aLanguageSymbol.
		ActiveWorld allTileScriptingElements do:
			[:viewerOrScriptor |
				viewerOrScriptor setNaturalLanguageTo: aLanguageSymbol]]! !

!Project methodsFor: 'language' stamp: 'sw 9/12/2001 23:03'!
stringForLanguageNameIs: aSym
	"Answer a string for a menu determining whether the given symbol is the project's natural language"

	^ (self naturalLanguage == aSym
		ifTrue: 	['<yes>']
		ifFalse:	['<no>']), ' ', aSym! !


!Project class methodsFor: 'utilities' stamp: 'sw 9/12/2001 23:05'!
chooseNaturalLanguage
	"Have the current project choose a new natural language"

	self current chooseNaturalLanguage! !


!TheWorldMenu methodsFor: 'construction' stamp: 'sw 9/13/2001 17:04'!
helpMenu
        "Build the help menu for the world."
        | screenCtrl genieEnabledString |

        screenCtrl _ ScreenController new.
	   genieEnabledString _ World currentHand isGenieEnabled ifTrue: ['disable'] ifFalse: ['enable'].
        ^ self fillIn: (self menu: 'help...') from:
        {
                {'about this system...'. {Smalltalk. #aboutThisSystem}. 'current version information.'}.
                {'update code from server'. {Utilities. #updateFromServer}. 'load latest code updates via the internet'}.
                {'preferences...'. {Preferences. #openPreferencesInspector}. 'view and change various options.'}.
			 {'set language...' . {Project. #chooseNaturalLanguage}. 'choose the language in which tiles should be displayed.'} .
                nil.
                {genieEnabledString , ' genie'. {World currentHand. #switchGenieEnabled}. genieEnabledString , ' gesture recognizer for the world''s current hand'}.
                {'genie gesture dictionaries'. {CRDictionary. #openInstanceBrowserMorph}. 'edit or inspect gesture dictionaries'.}.
                {'genie display properties'. {CRDisplayProperties. #openInstanceBrowserMorph}. 'edit or inspect display properies'.}.
                nil.
                {'command-key help'. { Utilities . #openCommandKeyHelp}. 'summary of keyboard shortcuts.'}.
                {'world menu help'. { self . #worldMenuHelp}. 'helps find menu items buried in submenus.'}.
                        "{'info about flaps' . { Utilities . #explainFlaps}. 'describes how to enable and use flaps.'}."
                {'font size summary' . { Utilities . #fontSizeSummary}.  'summary of names and sizes of available fonts.'}.
                {'useful expressions' . { Utilities . #openStandardWorkspace}. 'a window full of useful expressions.'}.
			 {'annotation setup...' . { Preferences . #editAnnotations}. 'Click here to get a little window that will allow you to specify which types of annotations, in which order, you wish to see in the annotation panes of browsers and other tools'}.
			nil.
                {'graphical imports' . { Smalltalk . #viewImageImports}.  'view the global repository called ImageImports; you can easily import external graphics into ImageImports via the FileList'}.
                {'standard graphics library' . { ScriptingSystem . #inspectFormDictionary}.  'lets you view and change the system''s standard library of graphics.'}.
                nil.
                {'telemorphic...' . {self. #remoteDo}.  'commands for doing multi-machine "telemorphic" experiments'}.
                {#soundEnablingString . { Preferences . #toggleSoundEnabling}. 'turning sound off will completely disable Squeak''s use of sound.'}.
                {'definition for...' . { Utilities . #lookUpDefinition}.  'if connected to the internet, use this to look up the definition of an English word.'}.
                nil.

                {'set author initials...' . { screenCtrl . #setAuthorInitials }. 'supply initials to be used to identify the author of code and other content.'}.
                {'vm statistics' . { screenCtrl . #vmStatistics}.  'obtain some intriguing data about the vm.'}.
                {'space left' . { screenCtrl . #garbageCollect}. 'perform a full garbage-collection and report how many bytes of space remain in the image.'}.
        }

! !


!TileMorph methodsFor: 'misc' stamp: 'sw 9/13/2001 21:58'!
currentEToyVocabulary
	"Answer the etoy vocabulary that pertains"

	| aVocab |
	^ ((aVocab _ self currentVocabulary) isKindOf: EToyVocabulary)
		ifTrue:
			[aVocab]
		ifFalse:
			[Vocabulary eToyVocabulary]! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/12/2001 23:05'!
setNaturalLanguageTo: aLangauge
	"With the natural language in the surrounding project having changed, get wording correct"

	self updateWordingToMatchVocabulary! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/13/2001 21:58'!
setSlotRefOperator: getter
	"getter represents the name of a slot that the receiver is to represent; configure the receiver to serve thi duty, and set upthe wording on the tile appropriately"

	type _ #operator.
	operatorOrExpression _ getter asSymbol.
	self line1:  (self currentEToyVocabulary tileWordingForSelector: operatorOrExpression).
	self updateLiteralLabel! !

!TileMorph methodsFor: 'private' stamp: 'sw 9/13/2001 21:58'!
updateLiteralLabel
	"Update the wording emblazoned on the tile, if needed"

	| myLabel |
	(myLabel _ self labelMorph) ifNil: [^ self].

	myLabel acceptValue:
		(type == #literal
			ifTrue:
				[literal] 
			ifFalse: [operatorReadoutString 
				ifNil:		[self currentEToyVocabulary tileWordingForSelector: operatorOrExpression]
				ifNotNil:  	[operatorReadoutString]]).
	self changed.! !


!AssignmentTileMorph methodsFor: 'initialization' stamp: 'sw 9/12/2001 22:49'!
computeOperatorOrExpression
	"Compute the operator or expression to use, and set the wording correectly on the tile face"

	| aSuffix wording anInterface getter doc |
	operatorOrExpression _ (assignmentRoot, assignmentSuffix) asSymbol.
	aSuffix _ self currentVocabulary translatedWordingFor:  assignmentSuffix.
	getter _ Utilities getterSelectorFor: assignmentRoot.
	anInterface _ self currentVocabulary methodInterfaceAt: getter ifAbsent: [Vocabulary eToyVocabulary methodInterfaceAt: getter ifAbsent: [nil]].
	wording _ anInterface ifNotNil: [anInterface elementWording] ifNil: [assignmentRoot copyWithout: $:].
	(anInterface notNil and: [(doc _ anInterface documentation) notNil])
		ifTrue:
			[self setBalloonText: doc].
	operatorReadoutString _ wording, ' ', aSuffix.
 	self line1: operatorReadoutString.
	self addArrowsIfAppropriate! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'sw 9/12/2001 22:52'!
updateWordingToMatchVocabulary
	"The current vocabulary has changed; change the wording on my face, if appropriate"

	self computeOperatorOrExpression! !


!Viewer methodsFor: 'commands' stamp: 'sw 9/6/2001 15:03'!
chooseVocabulary
	"Put up a menu allowing the user to specify which protocol to use in this viewer"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Choose a vocabulary'.
	"aMenu addStayUpItem."  "For debugging only"
	Vocabulary allStandardVocabularies do:
		[:aVocabulary |
			(scriptedPlayer class implementsVocabulary: aVocabulary)
				ifTrue:
					[aMenu add: aVocabulary vocabularyName selector: #switchToVocabulary: argument: aVocabulary.
					aVocabulary == self currentVocabulary ifTrue:
						[aMenu lastItem color: Color blue]. 
					aMenu balloonTextForLastItem: aVocabulary documentation]].
	aMenu popUpInWorld: self currentWorld! !


!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:16'!
adoptVocabulary: aVocabulary
	"Adopt the given vocabulary as the one used in this viewer."

	| aCategory |
	chosenCategorySymbol ifNil: [^ self delete].
	aCategory _ aVocabulary categoryAt: chosenCategorySymbol.
	aCategory ifNil: [self delete] ifNotNil: [self updateCategoryNameTo: aCategory wording].
	super adoptVocabulary: aVocabulary! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:55'!
categoryNameWhoseTranslatedWordingIs: aWording
	"Answer the category name with the given wording"

	| result |
	result _ self currentVocabulary categoryWhoseTranslatedWordingIs: aWording.
	^ result
		ifNotNil:
			[result categoryName]
		ifNil:
			[aWording]! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/12/2001 20:34'!
categoryWhoseTranslatedWordingIs: aWording
	"Answer the elementCategory with the given wording"

	^ self currentVocabulary categoryWhoseTranslatedWordingIs: aWording! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:59'!
chooseCategory
	"The mouse went down on my category-list control; pop up a list of category choices"

	| aList aMenu reply aLinePosition lineList |
	aList _ scriptedPlayer categoriesForViewer: self.

	aLinePosition _ aList indexOf: #miscellaneous ifAbsent: [nil].
	aList _ aList collect:	
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].

	lineList _ aLinePosition ifNil: [#()] ifNotNil: [Array with: aLinePosition].
	aMenu _ CustomMenu labels: aList lines: lineList selections: aList.
	reply _ aMenu startUpWithCaption: 'category'.
	reply ifNil: [^ self].
	self chooseCategoryWhoseTranslatedWordingIs: reply asSymbol
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:56'!
chooseCategoryWhoseTranslatedWordingIs: aWording
	"Choose the category with the given wording"

	self chosenCategorySymbol: (self categoryNameWhoseTranslatedWordingIs: aWording)
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:49'!
chosenCategorySymbol: aCategorySymbol
	"Make the given category be my current one."

	| aCategory wording |
	chosenCategorySymbol _ aCategorySymbol.
	aCategory _ self currentVocabulary categoryAt: chosenCategorySymbol.
	wording _ aCategory ifNil: [aCategorySymbol] ifNotNil: [aCategory wording].
	self categoryWording: wording! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/12/2001 22:58'!
isTileScriptingElement
	"Answer whether the receiver is a tile-scripting element"

	^ true! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:57'!
nextCategory
	"Change the receiver to point at the category following the one currently seen"

	| aList anIndex newIndex already aChoice |
	aList _ (scriptedPlayer categoriesForViewer: self) collect:
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].

	already _ self outerViewer ifNil: [#()] ifNotNil: [self outerViewer categoriesCurrentlyShowing].
	anIndex _ aList indexOf: self currentCategory ifAbsent: [0].
	newIndex _ anIndex = aList size
		ifTrue:		[1]
		ifFalse:		[anIndex + 1].
	[already includes: (aChoice _ aList at: newIndex)] whileTrue:
		[newIndex _ (newIndex \\ aList size) + 1].
	self chooseCategoryWhoseTranslatedWordingIs: aChoice! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:53'!
previousCategory
	"Change the receiver to point at the category preceding the one currently seen"

	| aList anIndex newIndex already aChoice |
	aList _ (scriptedPlayer categoriesForViewer: self) collect:
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].
	already _ self outerViewer ifNil: [#()] ifNotNil: [self outerViewer categoriesCurrentlyShowing].
	anIndex _ aList indexOf: self currentCategory ifAbsent: [aList size + 1].
	newIndex _ anIndex = 1
		ifTrue:		[aList size]
		ifFalse:		[anIndex - 1].
	[already includes: (aChoice _ aList at: newIndex)] whileTrue:
		[newIndex _ newIndex = 1 ifTrue: [aList size] ifFalse: [newIndex - 1]].

	self chooseCategoryWhoseTranslatedWordingIs: aChoice! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:50'!
secreteCategorySymbol
	"Set my chosenCategorySymbol by translating back from its representation in the namePane.  Answer the chosenCategorySymbol"

	| aCategory |
	aCategory _ self currentVocabulary categoryWhoseTranslatedWordingIs: self currentCategory.
	^ chosenCategorySymbol _ aCategory
		ifNotNil:
			[aCategory categoryName]
		ifNil:
			[self currentCategory]! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 20:42'!
setNaturalLanguageTo: aLanguage
	"Set the natural language symbol as indicated"

	chosenCategorySymbol ifNil: [^ self delete].
	self updateCategoryNameTo: (self currentVocabulary categoryWordingAt: chosenCategorySymbol)! !


!StandardViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 20:00'!
likelyCategoryToShow
	"Choose a category to show based on what's already showing and on some predefined heuristics"

	| possible all aCat currVocab |
	all _ (scriptedPlayer categoriesForViewer: self) asOrderedCollection.
	possible _ all copy.
	currVocab _ self currentVocabulary.
	self categoryMorphs do:
		[:m |
			aCat _ currVocab categoryWhoseTranslatedWordingIs:  m currentCategory.
			aCat ifNotNil: [possible remove: aCat categoryName ifAbsent: []]].

	(currVocab isKindOf: EToyVocabulary)  ifTrue: "hateful!!"
		[((possible includes: #'instance variables') and: [scriptedPlayer hasUserDefinedSlots])
			ifTrue:	[^ #'instance variables'].

		((possible includes: #scripts) and: [scriptedPlayer hasUserDefinedScripts])
			ifTrue:	[^ #'scripts']].

	#(basic #'color & border' color flagging comparing) do:
		[:preferred | (possible includes: preferred) ifTrue: [^ preferred]].
	^ possible size > 0
		ifTrue:
			[possible first]
		ifFalse:
			[all first]! !


!Vocabulary methodsFor: 'queries' stamp: 'sw 9/13/2001 14:29'!
atKey: aKey putMethodInterface: anInterface
	"Place the given interface at the given key."

	methodInterfaces at: aKey put: anInterface! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/12/2001 22:43'!
categoryWhoseTranslatedWordingIs: aWording
	"Answer the category whose translated is the one provided, or nil if none"

	^ self categories detect: [:aCategory | aCategory wording = aWording] ifNone: [nil]
! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/13/2001 19:52'!
categoryWordingAt: aSymbol
	"Answer the wording for the category at the given symbol"

	| result |
	result _ categories detect: [:aCategory | aCategory categoryName == aSymbol] ifNone: [nil].
	^ result
		ifNotNil:
			[result wording]
		ifNil:
			[aSymbol]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/12/2001 19:15'!
tileWordingForSelector: aSelector
	"Answer the wording to emblazon on tiles representing aSelector"

	| anInterface |
	anInterface _ self methodInterfaceAt: aSelector asSymbol ifAbsent:
		[^ (self translatedWordingFor: aSelector) ifNil: [Utilities inherentSelectorForGetter: aSelector]].
	^ anInterface elementWording! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 9/12/2001 14:53'!
addFromTable: aTable language: aLanguageSymbol
	"Add each method-specification tuples, each of the form:
		(1)	selector
		(2)	companion setter selector (#none or nil indicate none)
		(3)  argument specification array, each element being an array of the form
				<arg name>  <arg type>
		(4)  result type, (#none or nil indicate none)
		(5)  array of category symbols, i.e. the categories in which this element should appear.
		(6)  help message. (optional)
		(7)  wording (optional)
		(8)  auto update flag (optional) - if #updating, set readout to refetch automatically

	Make new categories as needed.
	Consult Vocabulary class.initializeTestVocabulary for an example of use"
				
	| aMethodCategory aMethodInterface aSelector doc wording |
	aTable do:
		[:tuple |   tuple fifth do: [:aCategorySymbol |
			(aMethodCategory _ self categoryAt: aCategorySymbol) ifNil: [ 
					aMethodCategory _ ElementCategory new categoryName: aCategorySymbol.
					self addCategory: aMethodCategory].		
		
			aMethodInterface _ MethodInterface new.
			aSelector _ tuple first.
			aMethodInterface selector: aSelector type: tuple fourth setter: tuple second.
			aMethodCategory elementAt: aSelector put: aMethodInterface.
			self atKey: aSelector putMethodInterface: aMethodInterface.
			((tuple third ~~ #none) and: [tuple third isEmptyOrNil not])
				ifTrue:
					[aMethodInterface argumentVariables: (tuple third collect:
						[:pair | Variable new name: pair first type: pair second])].

		
			doc _ (tuple size >= 6 and: [(#(nil none unused) includes: tuple sixth) not])
				ifTrue:
					[tuple sixth]
				ifFalse:
					[nil].
			wording _ (tuple size >= 7 and: [(#(nil none unused) includes: tuple seventh) not])
				ifTrue:
					[tuple seventh]
				ifFalse:
					[aSelector].
			aMethodInterface absorbTranslation: (ElementTranslation new wording: wording helpMessage: doc language: #English).
			tuple size >= 8 ifTrue:
				[aMethodInterface setToRefetch]]].
! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 9/12/2001 15:16'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	vocabularyName _ #unnamed.
	categories _ OrderedCollection new.
	methodInterfaces _ IdentityDictionary new! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 9/12/2001 15:26'!
initializeFromTable: aTable
	"Initialize the receiver from a list of method-specification tuples, each of the form:
		(1)	selector
		(2)	companion setter selector (#none or nil indicate none)
		(3)  argument specification array, each element being an array of the form
				<arg name>  <arg type>
		(4)  result type, (#none or nil indicate none)
		(5)  array of category symbols, i.e. the categories in which this element should appear.
		(6)  help message. (optional)
		(7)  wording (optional)
		(8)  auto update flag (optional) - if #updating, set readout to refetch automatically

	Consult Vocabulary class.initializeTestVocabulary for an example of use"
				
	|  aMethodCategory categoryList aMethodInterface aSelector doc wording |
	categoryList _ Set new.
	aTable do:
		[:tuple | categoryList addAll: tuple fifth].
	categoryList _ categoryList asSortedArray.
	categoryList do:
		[:aCategorySymbol |
			aMethodCategory _ ElementCategory new categoryName: aCategorySymbol.
			aTable do:
				[:tuple | (tuple fifth includes: aCategorySymbol) ifTrue:
					[aMethodInterface _ MethodInterface new.
					aSelector _ tuple first.
					aMethodInterface selector: aSelector type: tuple fourth setter: tuple second.
					aMethodCategory elementAt: aSelector put: aMethodInterface.
					self atKey: aSelector putMethodInterface: aMethodInterface.
					((tuple third ~~ #none) and: [tuple third isEmptyOrNil not])
						ifTrue:
							[aMethodInterface argumentVariables: (tuple third collect:
								[:pair | Variable new name: pair first type: pair second])].
					doc _ (tuple size >= 6 and: [(#(nil none unused) includes: tuple sixth) not])
						ifTrue:
							[tuple sixth]
						ifFalse:
							[nil].
 					wording _ (tuple size >= 7 and: [(#(nil none unused) includes: tuple seventh) not])
						ifTrue:
							[tuple seventh]
						ifFalse:
							[aSelector asString].
					aMethodInterface absorbTranslation: (ElementTranslation new wording: wording helpMessage: doc language: #English).
					tuple size >= 8 ifTrue:
						[aMethodInterface setToRefetch]]].
			self addCategory: aMethodCategory]! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/13/2001 14:11'!
addToTranslationTableFrom: categoryTriplets language: langSymbol
	"Create entries in my translationTable from catetgoryTriplets. Each triplet has: (key wording balloon-help)"

	| entryKey entry aTranslation |
	translationTable ifNil: [self initializeTranslationTable].
	categoryTriplets do:
		[:trip |
			entryKey _ trip first.
			entry _ translationTable elementAt: entryKey.
			aTranslation _ ElementTranslation fromPair: trip allButFirst language: langSymbol.
			entry ifNil:
				[translationTable elementAt: entryKey put: (entry _ ObjectWithDocumentation new)].
			entry absorbTranslation: aTranslation].
! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/13/2001 14:50'!
assureTranslationsAvailableFor: aLanguageSymbol
	"Any vocabulary that has translations for the language gets this called when language switches -- check reimplementors"! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/13/2001 13:14'!
initializeTranslationTable
	"initialize the translation table"

	translationTable _ ElementCategory new categoryName: #Translations.

"Vocabulary eToyVocabulary initializeTranslationTable"! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/12/2001 19:44'!
translateCategories: categoryTriplets language: aLanguage
	"Establish the category-name translations as per (internalCategorySymbol newCategoryWording balloon-help)"

	categoryTriplets do: [:aTriplet |
		categories do: [:aCategory |
			aCategory categoryName = aTriplet first
				ifTrue:
					[aCategory absorbTranslation: (ElementTranslation new wording: aTriplet second helpMessage: aTriplet third language: aLanguage)]]]! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/12/2001 16:18'!
translateMethodInterfaceWordings: wordingTriplets language: aLanguage
	"Go through my MethodInterfaces, translating wordings via wordingTriplets. Each triplet has: (selector wording balloon-help)"

	| wordingTable |
	wordingTable _ IdentityDictionary new.
	wordingTriplets do:
		[:trip | wordingTable at: trip first put: trip].
	methodInterfaces do:
		[:anInterface |
			(wordingTable at: anInterface selector ifAbsent: [nil]) ifNotNilDo:
				[:aTriplet |
					anInterface absorbTranslation:
						(ElementTranslation new wording: aTriplet second helpMessage: aTriplet third language: aLanguage)]]! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 9/13/2001 14:34'!
translatedWordingFor: aSymbol
	"If I have a translated wording for aSymbol, return it, else return aSymbol.  Caveat: at present, this mechanism is only germane for *assignment-operator wordings*"

	| translation |
	translationTable ifNil: [^ aSymbol].
	translation _ translationTable elementAt: aSymbol asSymbol.
	translation ifNil: [^ aSymbol].
	^ translation wording
! !


!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 16:36'!
addCustomCategoriesTo: categoryList
	"Add any further categories to the categoryList -- for benefit of subclasses wishing to override."! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 12:39'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	|   classes aMethodCategory selector selectors categorySymbols aMethodInterface |
	super initialize.
	self vocabularyName: #eToy.
	self documentation: '"EToy" is a vocabulary that provides the equivalent of the 1997-2000 etoy prototype'.
	categorySymbols _ Set new.
	classes _ self morphClassesDeclaringViewerAdditions.
	classes do:
		[:aMorphClass | categorySymbols addAll: aMorphClass basicNew categoriesForViewer].
	self addCustomCategoriesTo: categorySymbols.  "For benefit, e.g., of EToyVectorVocabulary"

	categorySymbols asOrderedCollection do:
		[:aCategorySymbol |
			aMethodCategory _ ElementCategory new categoryName: aCategorySymbol.
			selectors _ Set new.
			classes do:
				[:aMorphClass |
					 (aMorphClass additionsToViewerCategory: aCategorySymbol) do:
						[:anElement |
						aMethodInterface _ self methodInterfaceFrom: anElement.
						selectors add: (selector _ aMethodInterface selector).
						(methodInterfaces includesKey: selector) ifFalse:
							[methodInterfaces at: selector put: aMethodInterface].
						self flag: #deffered.
						"NB at present, the *setter* does not get its own method interface.  Need to revisit"].

			(selectors copyWithout: #unused) asSortedArray do:
				[:aSelector |
					aMethodCategory elementAt: aSelector put: (methodInterfaces at: aSelector)]].
				 
			self addCategory: aMethodCategory].

	#(scripts #'instance variables') do: [:sym | self addCategoryNamed: sym].
	self setCategoryDocumentationStrings.

	self addToTranslationTableFrom: #(
(:						'_'						'assign value')
(Incr:					'increase by'			'increase value by')
(Decr:					'decrease by'			'decrease value by')
(Mult:					'multiply by'			'multiply value by')) language: #English
! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 16:39'!
methodInterfaceFrom: elementTuple
	"Tedious revectoring:  The argument is a tuple of the sort that #additionsToViewerCategory: answers a list of; answer a MethodInterface"

	^ elementTuple first == #command
		ifTrue:
			[MethodInterface new initializeFromEToyCommandSpec: elementTuple category: nil]
		ifFalse:  "#slot format"
			[MethodInterface new initializeFromEToySlotSpec: elementTuple]! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 20:13'!
setCategoryDocumentationStrings
	"Initialize the documentation strings associated with the old etoy categories, in English"

	self translateCategories: #(
(basic					'basic'					'a few important things')
(#'book navigation'		'book navigation'		'relating to book, stacks, etc')
(button					'button'					'for thinking of this object as a push-button control')
(collections				'collections'				'for thinking of this object as a collection')
(fog					'fog'					'3D fog')
(geometry				'geometry' 				'measurements and coordinates')
(#'color & border'		'color & border'			'matters concerning the colors and borders of objects')
(graphics				'graphics'				'for thinking of this object as a picture')
(#'instance variables'	'instance variables'		'instance variables added by this object')
(joystick				'joystick	'				'the object as a Joystick')
(miscellaneous			'miscellaneous' 			'various commands')
(motion					'motion' 				'matters relating to moving and turning')
(paintbox				'paintbox'				'the painting palette')
(#'pen trails'			'pen trails'				'relating to trails put down by pens')
(#'pen use'				'pen use' 				'use of an object''s "pen"')
(playfield				'playfield'				'the object as a container for other visible objects')
(sampling				'sampling'				'sampling')
(scripts					'scripts'					'methods added by this object')
(slider					'slider'					'functions useful to sliders')
(speaker				'speaker'				'the object as an audio Speaker')
(#'stack navigation'		'stack navigation'		'navigation within a stck')
(storyboard				'storyboard'				'storyboard')
(tests					'tests'					'yes/no tests, to use in "Test" panes of scripts')
(text					'text'					'The object as text')
(vector					'vector'					'The object as a vector')
(viewing				'viewing'				'matters relating to viewing')
 ) language: #English
! !

!EToyVocabulary methodsFor: 'category list' stamp: 'sw 9/10/2001 08:50'!
categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass
	"Answer the category list for the given object, considering only code implemented in aClass and lower"

	^ (anObject isKindOf: Player)
		ifTrue:
			[self flag: #deferred.  "The bit commented out on next line is desirable but not yet workable, because it delivers categories that are not relevant to the costume in question"
			"#(scripts #'instance variables'), (super categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass)]"

			^ self translatedWordingsFor: ((mostGenericClass == aClass)
				ifFalse:
					[anObject categoriesForVocabulary: self]
				ifTrue:
					[#(scripts #'instance variables')])]
		ifFalse:
			[super categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass]! !

!EToyVocabulary methodsFor: 'method list' stamp: 'sw 9/13/2001 17:27'!
allMethodsInCategory: aCategoryName forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the etoy interface which are in the given category, on behalf of anObject, or if it is nil, aClass"

	| aCategory unfiltered suitableSelectors isAll |

	aCategoryName ifNil: [^ OrderedCollection new].
	aClass isUniClass ifTrue:
		[aCategoryName == #scripts ifTrue:
			[^ aClass namedTileScriptSelectors].
		aCategoryName == #'instance variables' ifTrue:
			[^ aClass slotInfo keys asSortedArray collect:
				[:anInstVarName | Utilities getterSelectorFor: anInstVarName]]].
	unfiltered _ (isAll _ aCategoryName = self allCategoryName)
		ifTrue:
			[methodInterfaces collect: [:anInterface | anInterface selector]]
		ifFalse:
			[aCategory _ categories detect: [:cat | cat categoryName == aCategoryName] 
							ifNone: [^ OrderedCollection new].
			aCategory elementsInOrder collect: [:anElement | anElement selector]].

	(anObject isKindOf: Player) ifTrue:
		[suitableSelectors _ anObject costume selectorsForViewer.
		unfiltered _ unfiltered  select:
			[:aSelector | suitableSelectors includes: aSelector]].
	(isAll and: [aClass isUniClass]) ifTrue:
		[unfiltered addAll: aClass namedTileScriptSelectors.
		unfiltered addAll: (aClass slotInfo keys asSortedArray collect:
			[:anInstVarName | Utilities getterSelectorFor: anInstVarName])].

	^ (unfiltered copyWithoutAll: #(dummy unused)) asSortedArray! !

!EToyVocabulary methodsFor: 'language translations' stamp: 'sw 9/13/2001 14:49'!
addGermanVocabulary
	"Add a German etoy vocabulary"
	self translateMethodInterfaceWordings: #(

(append:					'hnge an'				'Fgt ein Objekt in diesen Behlter ein')
(beep:						'mache Gerusch'  				'Macht das angegebene Gerusch')
(bounce:					'pralle ab'					'Lt das Objekt vom Rand seines Behlters abprallen und spielt das angegebene Gerusch, wenn es auerhalb ist')
(cameraPoint 				'Kamerapunkt'				'Der Kamerapunkt des Objektes') 
(clear 						'lsche Graph'						'Lscht den momentanen Graphen')
(clearOwnersPenTrails		'lsche Stiftspuren im Eigner'	'Lscht alle Stiftspuren im Eigner')
(clearTurtleTrails 			'lsche Stiftspuren' 		'Lscht all Stiftspuren im Objekt')
(color:sees: 					'Farbe sieht'			'berprft, ob die angebene Farbe des Objektes die Testfarbe sehen kann')
(deleteCard 					'lsche Karte'				'Lscht diese Karte aus dem Stapel')
(doMenuItem: 				'fhre Menpunkt aus'	'Fhrt den angebenen Menpunkt aus')
(emptyScript 				'leeres Skript'				'Ein leeres Skript')
(fire						'feuer'						'Fhrt alle zugehrigen Aktionen dieses Schalters aus')
(firstPage 					'gehe zur ersten Seite'		'Geht zur ersten Seite')
(followPath					'folge Pfad'					'Folge dem definierten Pfad')
(forward: 					'gehe vorwrts um'		'Bewegt das Objekt vorwrts in seiner momentanen Richtung')
(goToFirstCardInBackground	'gehe zur ersten Karte im Hintergund'	'Geht zur ersten Karte im momentanen Hintergrund')
(goToFirstCardOfStack		'gehe zur ersten Karte im Stapel'	'Geht zur ersten Karte im Stapel')
(goToLastCardInBackground	'gehe zur letzten Karte im Hintergund'	'Geht zur letzten Karte im momentanen Hintergrund')
(goToLastCardOfStack		'gehe zur letzten Karte im Stapel'		'Geht zur letzten Karte im Stapel')
(goToNextCardInStack		'gehe zur nchsten Karte im Stapel'		'Geht zur nchsten Karte im momentanen Hintergrund')
(goToPreviousCardInStack	'gehe zur vorherigen Karte im Stapel'	'Geht zur vorherigen Karte im momentanen Hintergrund')
(goToRightOf:				'plaziere rechts von'		'Setzt das Objekt rechts neben ein anderes')
(goto:						'gehe zur Seite'		'Geht zur angegebenen Seite')
(hide					'verstecke Dich'					'Versteckt das Objekt')
(initiatePainting			'beginne neue Zeichnung'		'Beginnt eine neue Zeichnung')
(insertCard				'erzeuge Karte'				'Erzeugt eine neue Karte und fgt sie ein.')
(lastPage				'gehe zur letzten Seite'			'Geht zur letzten Seite')
(liftAllPens				'nimm alle Stifte hoch'			'Nimmt alle Stifte hoch, die sich im Inneren befinden')
(loadSineWave			'lade Sinuswelle'			'Ldt eine Sinuswelle als momentanen Graph')
(loadSound:				'lade Gerusch'					'Ldt das angegebene Gerusch als momentanen Graph')
(lowerAllPens			'setze alle Stifte ab'			'Setzt alle Stifte ab, die sich im Inneren befinden')
(makeNewDrawingIn:	'beginne neue Zeichnung in'	'Beginnt eine neue Zeichnung im angegebenen Objekt')
(moveToward:			'gehe in Richtung '				'Bewegt das Objekt in Richtung eines anderen Objektes')
(nextPage				'gehe zur nchsten Seite'		'Geht zur nchsten Seite')
(pauseScript:			'stoppe Skript'					'Hlt ein Skript an')
(play					'spiele Gerausch ab'				'Spielt den momentanen Graphen als Gerusch ab')
(previousPage			'gehe zur vorherigen Seite'	'Geht zur vorherigen Seite')
(removeAll				'entferne alles'				'Entfernt und lscht alle Elemente')
(reverse				'umdrehen'						'Dreht den Inhalt des Graphen um')
(roundUpStrays			'hole zurck'					'Holt alle Objekte zurck, falls sie sich irgendwo verstecken')
(seesColor:				'sieht die Farbe'					'berprft, ob das Objekt die angegebene Farbe sieht')
(show					'zeige Dich'						'Zeigt das Objekt')
(shuffleContents			'mische Inhalt'					'Mischt alle Objekte zufllig')
(stampAndErase			'stanze und lsche Dich'		'Fgt das Abbild des Objektes den Stiftspurent hinzu und lscht es anschlieend.')
(startScript:				'starte Skript'					'Beginnt die wiederholte Ausfhrung eines Skriptes')
(stopScript:				'stoppe Script'				'Beendet die wiederholte Ausfhrung eines Skriptes')
(tellAllSiblings:			'sage den Geschwistern'		'Sendet eine Nachricht zu allen Geschwistern des Objektes')
(touchesA:				'berhrt'					'berprft, ob ein Objekt des angegebenen Typs berhrt wird')
(turn:					'drehe Dich um'				'ndert die Richtung des Objektes um den angegebenen Winkel')
(unhideHiddenObjects	'zeige versteckte Objekte' 		'Zeigt alle versteckten Objekte an')
(wearCostumeOf: 		'trage Kostm von'				'Trgt das Kostm eines anderen Objektes')
(wrap					'wickel Dich rum'				'Wickelt das Objekt um den Rand seines Behlters')

(getActWhen 			'Ausfhrungsstatus'	'Bestimmt, wann das Skript ausgefhrt wird')
(getAllButFirstCharacter	'Alle Buchstaben auer dem Ersten' 'Enthlt alle Buchstaben auer dem Ersten')
(getAmount				'Betrag'					'Der Betrag der Abweichung vom Zentrum')
(getAngle				'Winkel'					'Der Winkel der Abweichung vom Zentrum')
(getBorderColor			'Randfarbe'		'Die Farbe des Randes')
(getBorderWidth			'Randbreite'		'Die Breite des Randes')
(getBottom 				'untere Kante'					'Die untere Kante des Objektes')
(getBrightnessUnder		'Helligkeit darunter'			'Die Helligkeit unter dem Zentrum des Objektes')
(getCharacters			'Buchstaben'					'Die Buchstaben des Inhalts')
(getColor				'Farbe' 					'Die Farbe des Objektes')
(getColorUnder 			'Farbe darunter'			'Die Farbe unter dem Zentrum des Objektes')
(getConePosition			'Membranposition'			'Die Position der Membran des Lautsprechers')
(getCursor				'Zeiger'					'Der momentane Zeigerindex')
(getDescending			'Absteigend'				'Bestimmt, ob der kleinste Wert zuerst gezeigt werden soll')
(getDistance				'Distanz'				'Die Distanz zum Ursprung des Behlters')
(getFirstCharacter		'Erster Buchstabe'		'Der erste Buchstabe des Inhalts')
(getFirstElement			'Erstes Element'			'Das erste Element des Inhalts')
(getFogColor				'Nebelfarbe'				'Die Farbe des Nebels')
(getFogDensity			'Nebeldichte'			'Die Dichte des Nebels')
(getFogRangeEnd		'Nebelanfang'			'Die Anfangsdistanz des Nebels')
(getFogRangeStart		'Nebelende'				'Die Enddistanz des Nebels')
(getFogType				'Nebeltyp'				'Der Typ des Nebels')
(getGraphic				'Bild'					'Das Bild des Objektes')
(getGraphicAtCursor		'Bild am Zeiger'			'Das Bild des Objektes am Zeiger')
(getHeading				'Richtung'				'Die Richtung in die das Objekt weist')
(getHeight				'Hhe'					'Die Hhe des Objektes')
(getHolder				'Behlter'				'Der Behlter dieses Objekt')
(getIndexInOwner		'Eignerindex' 			'Der Index des Objektes in seinem Eigner')
"@@@: Folgendes sollte vermutlich die Hand und nicht die Maus referenzieren :@@@"
(getIsUnderMouse		'ist Maus darber'		'berprft, ob die Maus ber dem Objekt ist')
"@@@: Sollte vielleicht 'Griff' heissen, aber ich mag Knubbel :-) :@@@"
(getKnobColor			'Knubbelfarbe'			'Die Farbe des Knubbels')
(getLabel				'Beschriftung'			'Die Beschriftung des Objektes')
(getLastValue			'letzter Wert'			'Der letzte Eingabewert')
(getLeft					'linke Kante'			'Die linke Kante des Objektes')
(getLeftRight			'links-rechts'			'Die horizontale Abweichung vom Zentrum')
(getLuminanceUnder	'Leuchtkraft darunter'	'Die Leuchtkraft unter dem Objekt')
(getMaxVal				'Maximalwert'			'Der maximale Wert des Reglers')
(getMinVal				'Minimalwert'			'Der minimale Wert des Reglers')
(getMouseX				'Maus X-Position'		'Die X Koordinate der Mausposition')
(getMouseY				'Maus Y-Position'		'Die Y Koordinate der Mausposition')
(getNewClone			'kopiere Dich'			'Erzeugt eine Kopie des Objektes')
(getNumberAtCursor		'Zahl am Zeiger'			'Die Zahl am Zeiger')
(getNumericValue		'Reglerwert'				'Der momentane Wert des Reglers')
(getObtrudes				'ragt hinaus'			'berprft, ob das Objekt aus seinem Eigner herausragt')
(getPenColor				'Stiftfarbe'				'Die Farbe des Stiftes')
(getPenDown			'Stift unten'				'Der Status des Stiftes')
(getPenSize				'Stiftgre'			'Der Durchmesser des Stiftes')
(getRight				'rechte Kante'			'Die rechte Kante des Objektes')
(getRoundedCorners		'runde Ecken'			'Bestimmt, ob Ecken abgerundet werden')
(getSampleAtCursor		'Stichprobe'				'Eine Stichprobe des Wertes an der momentanen Zeigerposition')
(getSaturationUnder		'Sttigung darunter'	'Die Sttigung der Farbe unter dem Objekt')
(getScaleFactor			'Skalierungsfaktor'		'Der Skalierungsfaktor des Objektes')
(getTheta				'Theta'					'Der ''Zentrums-Ursprungs-Rechts'' Winkel (huh?)')
(getTop					'obere Kante'			'Die obere Kante des Objekts')
(getTruncate			'Abrunden'				'Bestimmt, ob nur ganze Zahlen benutzt werden')
(getUpDown				'hoch-runter'			'Die vertikale Abweichung vom Zentrum')
(getValueAtCursor		'Objekt am Zeiger'		'Das Objekt an der momentanen Zeigerposition')
(getViewingByIcon 		'Symboldarstellung'		'Bestimmt, ob Objekte symbolisch oder normal dargestellt werden')
(getX					'x'						'Die X Koordinate des Objektes')
(getY					'y'						'Die Y Koordinate des Objektes')
(getWidth				'Breite'					'Die Breite des Objektes')) language: #Deutsch.


	self translateCategories: #(
(basic					Einfach					'Standardsicht')
(#'book navigation'		#'Buchnavigation'		'Navigation in Bchern')
(button					Schalter					'Das Objekt als Schalter')
(collections				Behlter				'Das Objekt als Behlter')
(fog					Nebel					'3D Nebel Eigenschaften')
(geometry				Geometrie				'Zur Geometrie des Objektes')
(#'color & border'		#'Farbe & Rand'			'Zum Thema Farben und Rnder')
(graphics				Graphik					'Graphische Eigenschaften')
(#'instance variables'	Variablen				'Variablen des Objektes')
(joystick				Joystick					'Das Objekt als Joystick')
(miscellaneous			Verschiedenes			'Alles was woanders nicht hinpat')
(motion					Bewegung				'Bewegungseigenschaften')
(paintbox				Malpalette				'Die Zeichenpalette')
(#'pen trails'			Stiftspuren				'Alles zum Thema Spuren hinterlassen')
(#'pen use'				Stifte					'Verwendung von Stiften')
(playfield				Spielfeld				'ja ich wei auch nicht...')
(sampling				Messen					'Messungen von Werten')
(scripts					Skripte					'Alle Deine Skripte')
(slider					Regler					'Das Objekt als Regler')
(speaker				Lautsprecher			'Das Objekt als Lautsprecher')
(#'stack navigation'		Stapelnavigation		'Navigation in Stapeln')
(storyboard				Storyboard				'Spter mal...')
(tests					Tests					'Verschiedene Tests')
(text					Text						'Das Objekt als Text')
(viewing				Betrachtung				'Wie man''s so sieht')
(vector					Vektor					'Das Objekt als Vektor')
 ) language: #Deutsch
! !

!EToyVocabulary methodsFor: 'language translations' stamp: 'sw 9/13/2001 20:12'!
addKiswahiliVocabulary
	"Add a Kiswahili vocabulary"

	self translateMethodInterfaceWordings: #(

(append:					'tia mwishoni'				'weka kitu hicho mwishoni')
(beep:						'fanya kelele'  				'piga kelele fulani')
(bounce:					'ruka duta'					'ruka duta kama mpira')
(cameraPoint 				'penye kamera'				'mahali penya kamera') 
(clear 						'kumba'						'ondoa vilivyokwemo')
(clearOwnersPenTrails		'ondoa nyayo'				'ondoa nyayo za wino')
(clearTurtleTrails 			'ondoa nyayo ndani' 		'ondoa nyayo za wino zilzo ndani')
(color:sees: 					'rangi yaona rangi'			'kama rangi fulana yaona rangi nyingine')
(deleteCard 					'tupa karata'				'tupa karata hii')
(doMenuItem: 				'fanya uchaguzi'			'fanya uchaguzi fulani')
(doScript:					'piga script'					'piga script ya jina fulani mara moja')
(emptyScript 				'script tupu'				'tengeneza script mpya tupu')
(fire						'waka'						'waka script, yaani kuianzisha')
(firstPage 					'nenda mwanzoni'			'nenda penye ukurasa wa kwanza')
(followPath					'fuata njia'					'fuata njia iliyofanywa kabla')
(forward: 					'nenda mbele'				'sogea mbela kwa kiasi fulani')
(goToFirstCardInBackground	'endea kwanza ya nyuma'	'endea karata kwanza ya nyuma')
(goToFirstCardOfStack		'endea kwanza ya stack'	'endea karata iliyo ya kwanza ya stack')
(goToLastCardInBackground	'endea mwisho ya nyuma'	'endea karata ya mwisho ya nyuma')
(goToLastCardOfStack		'endea mwisho ya stack'		'endea karata ya mwisho ya stack')
(goToNextCardInStack		'endea karata ifuatayo'		'endea karata itakayofuata penye stack')
(goToPreviousCardInStack	'endea karata itanguliayo'	'endea karata kliyonitangulia penye stack')
(goToRightOf:				'endea karibu ya kulia'		'sogea hata nipo upande wa kulia kuhusu kitu fulani')
(goto:						'endea mahali fulani'		'endea mahali fulani')
(hide					'ficha'							'nifanywe ili nisionekane')
(initiatePainting			'anza kupiga picha'				'anza kupiga picha mpya')
(insertCard				'weka karata mpya'				'weka karata mpya ndani ya stack')
(lastPage				'ukurasa wa mwisho'			'endea ukurasa ya mwisho')
(liftAllPens				'inua kalamu zote'				'inua kalamu zote zilizomo ndani, ili zisipige rangi')
(loadSineWave			'pakia wimbi la sine'			'pakia wimibi (la kitrigonometry) la sine')
(loadSound:				'pakia kelele'					'pakia kelele fulani')
(lowerAllPens			'telemsha kalamu zote'			'telemsha kalamu zote ya vitu vyote vilivyomo ndani')
(makeNewDrawingIn:	'anza kupiga picha kiwanjani'	'anza kupaga picha mpya ndani ya kiwanja')
(moveToward:			'nenda upande wa'				'nenda upande wa kitu fulani')
(nextPage				'endea ukurasa ufuatao'		'nenda ukurasani unaofuata')
(pauseScript:			'pumzisha script'				'pumzisha script fulani')
(pauseAll:				'pumzisha script zote'			'pumzisha script fulani katika mwenyewe na ndugu zangu wote')
(play					'cheza'							'cheza, basi!!')
(previousPage			'endea ukurasa uliotangulia'	'enda ukurasa uliotangulia ukurusa huu')
(removeAll				'ondoa vyote vilivyokuwemo'	'ondoa vitu vyote vilvyomo dani')
(reverse				'kinyume'						'kinyume cha upande')
(roundUpStrays			'kusanya'						'sanya vitu vilovyopotoleka')
(seesColor:				'yaona rangi'					'kama naona rangi fulani')
(show					'onyesha'						'fanya hata naonekana')
(shuffleContents			'changanya'					'changanya orodha ya ndani')
(stampAndErase			'piga chapa na kufuta'			'piga chapa, halafu kufuta')
(startScript:				'anzisha script'					'anzisha script ya jina fulani')
(startAll:				'anzisha script zote'				'anzisha script fulani penye mwenyewe na ndugu zangu wote')
(stopAll:					'simamisha script zote'			'simamisha script fulani penye mwenyewe na ndugu zangu wote')
(stopScript:				'simamisa skriptu'				'simamisha script ya jana fulani')
(tellAllSiblings:			'watangazie ndugu'				'tangaza habari kwa ndugu zangu wote')
(touchesA:				'yagusa'						'kama nagusa kitu cha aina fulani')
(turn:					'geuka'							'geuka kwa pembe fulani')
(unhideHiddenObjects	'onyesha vilivyofichwa' 		'onyesha vitu ndani vilivyofichwa')
(wearCostumeOf: 		'vaa nguo za'					'vaa nguo za mtu mwingine')
(wrap					'zunguka'						'baada ya kutoka, ingia n''gambo')

(getActWhen 			'waka kama'			'lini ya waka')
(getAllButFirstCharacter	'herufi ila ya kwanza' 'herufi zote isipokuwa ile ya kwanza tu')
(getAmount				'kiasi'					'kiasi gani')
(getAngle				'pembe'					'pembe iliyopo (degree)')
(getBorderColor			'rangi ya mpaka'		'rangi ya mpaka wangu')
(getBorderWidth			'upana wa mpaka'		'upana wa mpaka wangu')
(getBottom 				'chini'					'chini yangu')
(getBrightnessUnder		'mng''aro chini'			'mwangaza chini yangu')
(getCharacters			'herufi'					'herufi zangu')
(getColor				'rangi' 					'rangi yangu')
(getColorUnder 			'rangi chini'			'rangi chini yangu')
(getConePosition			'penye cone'			'mahali penye cone')
(getCursor				'kidole'					'namba ya kitu ndani kilichagulwa')
(getDescending			'kama yaenda chini'	'kama naonyesha vitu chini')
(getDistance				'urefu'					'urefu kutoka asili')
(getFirstCharacter		'herufi ya kwanza'		'herufi yangu ya kwanza')
(getFirstElement			'kitu cha kwanza'		'kitu changu cha ndani cha kwanza')
(getFogColor				'rangi ya ukungu'		'rangi ya ukungu wangu')
(getFogDensity			'nguvu wa ukungu'	'nguvu ya ukungu wangu')
(getFogRangeEnd		'mwisho wa ukungu'	'mwisho wa upana wa ukungu wangu')
(getFogRangeStart		'mwanzo wa ukungu'	'mwanzo wa upana wa ukungu wangu')
(getFogType				'aina ya ukungu'		'aina ya ukungu wangu')
(getGraphic				'picha'					'picha ninayonyesha')
(getGraphicAtCursor		'picha penye kidolee'	'picha iliyopo penye kidole changu')
(getHeading				'upande'				'upande gani ninayoelekea')
(getHeight				'urefu'					'urefu wangu')
(getHolder				'mshikoi'				'kitu niliomo ndani yake')
(getIndexInOwner		'namba kataki mwenyeji' 'namba niliyo nayo katika mwenyeji')
(getIsUnderMouse		'chini kipanya'			'kama nipo chini ya kipanya')
(getKnobColor			'rangi ya ndani'		'rangi ya sehemu yangu ya ndani')
(getLabel				'tangazo'				'iliyoandishwa juu yangu')
(getLastValue			'mapimo'				'iliyokuwemo ndani')
(getLeft					'kushoto'				'mpaka wa kushoto')
(getLeftRight			'kiasi cha sawasawa'	'kiasi cha kushoto ama kulia')
(getLuminanceUnder	'uNg''aa chini'			'uNg''aa ya sehemu chini yangu')
(getMaxVal				'kiasi cha juu'			'kiasi cha juu humu ndani')
(getMinVal				'kiasi cha chini'		'kiasi cha chini humu ndani')
(getMouseX				'x ya kipanya'			'mahali pa x pa kipanya')
(getMouseY				'y ya kipanya'			'mahali pa y pa kipanya')
(getNewClone			'nakala'				'fanya nakala yangu')
(getNumberAtCursor		'namba kidoleni'		'namba iliyopo kidoleni')
(getNumericValue		'namba humu'			'namba iliyopo katika kituc hicho')
(getObtrudes				'jiingiliza'				'kama kitu hicho hujiingiliza')
(getPenColor				'rangi ya kalamu'		'rangi ninayotumia kwa kalamu')
(getPenDown			'kalamu chini'			'kama kalamu hukaa chini')
(getPenSize				'upana wa kalamu'		'urefu wa kalamu ninayotumia')
(getRight				'kulia'					'mpaka wa kulia')
(getRoundedCorners		'viringisha'				'tumia pembe zilizoviringishwa')
(getSampleAtCursor		'kiasi kidoleni'			'kiasi kilichopo kidoleni')
(getSaturationUnder		'kunyewesha chini'		'kiasi cha kunyewesha chini ya kati yangu')
(getScaleFactor			'kuzidisha kwa'			'kiasi ninachozidishwa nacho')
(getTheta				'theta'					'pemba kwa x-axis')
(getTop					'juu'					'mpaka wa juu')
(getTruncate			'kata'					'kama kukata ama sivyo')
(getUpDown				'juu/chini'				'kiasi cha juu ama cha chini')
(getValueAtCursor		'mchezaji kidoleni'		'mchechazji aliyepo kidoneni')
(getViewingByIcon 		'angalia kwa picha'		'kama vitu vilivyomo ndani huanagaliwa kwa picha ama sivyo')
(getX					'x'						'mahali pa x')
(getY					'y'						'mahali ya y')
(getWidth				'upana'					'upana wangu')) language: #Kiswahili.

	self translateCategories: #(
(basic					muhimu				'mambo muhimu muhimu')
(#'book navigation'		#'kuongoza vitabu'		'kuhusu kuongozea vitabu')
(button					kifungo					'mambo kuhusu vifungo')
(collections				mikusanyo				'kuhusu mikusanyo ya vitu')
(fog					ukungu					'kuhusu ukungu (3D)')
(geometry				kupimia					'urefu na kadhaliki')
(#'color & border'		#'rangi & mpaka'		'kuhusu rangi na mpaka')
(graphics				picha					'mambo kuhusu picha')
(#'instance variables'	badiliko					'data zilizoundwa na yule atumiaye')
(joystick				#'fimbo la furaha'		'kuhusu fimbo la furaha, yaani "joystick"')
(miscellaneous			mbalimbali				'mambo mbalimbali')
(motion					kusogea					'kwenda, kuegeuka, etc.')
(paintbox				#'kupiga rangi'			'vitu kuhusu kupigia rangi')
(#'pen trails'			#'nyayo za kalamu'		'kuhusu nyay za kalamu')
(#'pen use'				#'kalamu'				'kuhusu kalamu')
(playfield				kiwanja					'vitu kuhusu kiwanjani')
(sampling				kuchagua				'mambo kuhusu kuchagua')
(scripts					taratibu				'taratibu zilizoundwa na atumiaye')
(slider					telezo					'kitu kionyeshacho kiasi cha namba fulani')
(speaker				spika					'kuhusu spika za kelele')
(#'stack navigation'		#'kuongoza chungu'	'kuhusu kuongozea chungu')
(storyboard				kusimulia				'kusimilia hadithi')
(tests					kama					'amua kama hali fulani i kweli ama sivyo')
(text					maneno					'maandiko ya maneno')
(viewing				kuangaliwa				'kuhusu kuangalia vitu')
(vector					vektor					'kuhusu vektor')
) language: #Kiswahili.

self addToTranslationTableFrom: 

#((:						'_'						'tia ndani')
(Incr:					'pamoja na'				'tia thamani + fulani')
(Decr:					'toa'					'tia thamani - fulani')
(Mult:					'mara'					'tia thamani * fulani')) language: #Kiswahili.
! !

!EToyVocabulary methodsFor: 'language translations' stamp: 'sw 9/13/2001 16:38'!
assureTranslationsAvailableFor: aLanguageSymbol
	"Any vocabulary that has translations for the language gets this called when language switches -- check reimplementors"

	aLanguageSymbol == #Deutsch ifTrue: [self addGermanVocabulary].
	aLanguageSymbol == #Kiswahili ifTrue: [self addKiswahiliVocabulary].! !

!EToyVocabulary methodsFor: 'language translations' stamp: 'sw 9/13/2001 18:41'!
templateForLanguageTranslation
	
"Edit this method such that the second element of each triplet has the translated wording and the third element has the translated help-message; give the edited method a name of the form #addLangVocabulary, and be sure to change the language name in the three places that it occurs, as #YourLanguage, below.

A complete translation consists, as in #addKiswahiliVocabulary, of calls to three methods, namely:
	translateMethodInterfaceWordings:language: 
	translateCategories:language:
	addToTranslationTableFrom:language: 

After editing this method into the one that holds your language translations, the next step is to edit #assureTranslationsAvailableFor: so that it calls the method you just created when appropriate.   Consult #addKiswahiliVocabulary and its sender for a complete example to emulate."

	self translateMethodInterfaceWordings: #(
(append: 'append' 'Add the object to my content')
(beep: 'make sound' 'Make the specified sound')
(bounce: 'bounce' 'bounce off the edge if hit')
(cameraPoint #cameraPoint 'the camera point')
(clear 'clear' 'Clear the graph of current contents')
(clearOwnersPenTrails 'clear all pen trails' 'clear all pen trails in my containing playfield')
(clearTurtleTrails 'clear pen trails' 'Clear all the pen trails in the interior.')
(color:sees: #colorSees 'whether the given color sees the given color')
(deleteCard 'deleteCard' 'Delete the current card')
(doMenuItem: 'do menu item' 'do the menu item')
(doScript: 'do' 'run the given script once, on the next tick')
(emptyScript 'emptyScript' 'an empty script')
(fire 'fire' 'trigger any and all of this object''s button actions')
(firstPage 'firstPage' 'go to first page')
(followPath 'followPath' 'follow the yellow brick road')
(forward: 'forward by' 'Moves the object forward in the direction it is heading')
(getActWhen #actWhen 'When the script should fire')
(getAllButFirstCharacter #allButFirst 'All my characters except the first one')
(getAmount #amount 'The amount of displacement')
(getAngle #angle 'The angular displacement')
(getBorderColor #borderColor 'The color of the object''s border')
(getBorderWidth #borderWidth 'The width of the object''s border')
(getBottom #bottom 'The bottom edge')
(getBrightnessUnder #brightnessUnder 'The brightness under the center of the object')
(getCharacters #characters 'The characters in my contents')
(getColor #color 'The color of the object')
(getColorUnder #colorUnder 'The color under the center of the object')
(getConePosition #conePosition 'the position of the speaker cone')
(getCursor #cursor 'The current cursor location, wrapped back to the beginning if appropriate')
(getDescending #descending 'Tells whether the smallest value is at the top/left (descending = false) or at the bottom/right (descending = true)')
(getDistance #distance 'The length of the vector connecting the origin to the object''s position')
(getFirstCharacter #firstCharacter 'The first character in my contents')
(getFirstElement #firstElement 'The first object in my contents')
(getFogColor #fogColor 'The color of fog being applied')
(getFogDensity #fogDensity 'The density of fog being applied')
(getFogRangeEnd #fogRangeEnd 'The range start of fog being applied')
(getFogRangeStart #fogRangeStart 'The range start of fog being applied')
(getFogType #fogType 'The type of fog being applied')
(getGraphic #graphic 'The picture currently being worn')
(getGraphicAtCursor #graphicAtCursor 'the graphic worn by the object at the cursor')
(getHeading #heading 'Which direction the object is facing.  0 is straight up')
(getHeight #height 'The height')
(getHolder #holder 'the object''s container')
(getIndexInOwner #elementNumber 'my index in my container')
(getIsUnderMouse #isUnderMouse 'whether the object is under the current mouse position')
(getKnobColor #knobColor 'The color of the slider')
(getLabel #label 'The wording on the button')
(getLastValue #lastValue 'The last value obtained')
(getLeft #left 'The left edge')
(getLeftRight #leftRight 'The horizontal displacement')
(getLuminanceUnder #luminanceUnder 'The luminance under the center of the object')
(getMaxVal #maxVal 'The number represented when the knob is at the right or bottom of the slider; the largest value returned by the slider.')
(getMinVal #minVal 'The number represented when the knob is at the left or top of the slider; the smallest value returned by the slider.')
(getMouseX #mouseX 'The x coordinate of the mouse pointer')
(getMouseY #mouseY 'The y coordinate of the mouse pointer')
(getNewClone #copy 'returns a copy of this object')
(getNumberAtCursor #numberAtCursor 'the number at the cursor')
(getNumericValue #numericValue 'A number representing the current position of the knob.')
(getObtrudes #obtrudes 'whether the object sticks out over its container''s edge')
(getPenColor #penColor 'the color of ink used by the pen')
(getPenDown #penDown 'whether the pen is currently down')
(getPenSize #penSize 'the width of the pen')
(getRight #right 'The right edge')
(getRoundedCorners #roundedCorners 'Whether corners should be rounded')
(getSampleAtCursor #sampleAtCursor 'The sample value at the current cursor location')
(getSaturationUnder #saturationUnder 'The saturation under the center of the object')
(getScaleFactor #scaleFactor 'The factor by which the object is magnified')
(getTheta #theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position')
(getTop #top 'The top edge')
(getTruncate #truncate 'If true, only whole numbers are used as values; if false, fractional values are allowed.')
(getUpDown #upDown 'The vertical displacement')
(getValueAtCursor #playerAtCursor 'the object currently at the cursor')
(getViewingByIcon #viewingNormally 'whether contents are viewed normally')
(getWidth #width 'The width')
(getX #x 'The x coordinate')
(getY #y 'The y coordinate')
(goToFirstCardInBackground 'goToFirstCardInBackground' 'Go to the first card of the current background')
(goToFirstCardOfStack 'goToFirstCardOfStack' 'Go to the first card of the entire stack')
(goToLastCardInBackground 'goToLastCardInBackground' 'Go to the last card of the current background')
(goToLastCardOfStack 'goToLastCardOfStack' 'Go to the last card of the entire stack')
(goToNextCardInStack 'goToNextCardInStack' 'Go to the next card')
(goToPreviousCardInStack 'goToPreviousCardInStack' 'Go to the previous card')
(goToRightOf: 'align after' 'place this object to the right of another')
(goto: 'goto:' 'go to the given page')
(hide 'hide' 'make the object invisible')
(initiatePainting 'initiatePainting' 'Initiate painting of a new object in the standard playfield.')
(insertCard 'insertCard' 'Create a new card')
(lastPage 'lastPage' 'go to last page')
(liftAllPens 'lift all pens' 'Lift the pens on all the objects in my interior.')
(loadSineWave 'loadSineWave' 'Load a sine wave as the current graph')
(loadSound: 'loadSound:' 'Load the specified sound into the current graph')
(lowerAllPens 'lower all pens' 'Lower the pens on all the objects in my interior.')
(makeNewDrawingIn: 'start painting in' 'make a new drawing in the specified playfield')
(moveToward: 'move toward' 'move toward the given object')
(nextPage 'nextPage' 'go to next page')
(pauseAll: 'pauseAll:' 'make the given script be "paused" in the object and all of its siblings')
(pauseScript: 'pause script' 'make the given script be "paused"')
(play 'play' 'Play the current graph as a sound')
(previousPage 'previousPage' 'go to previous page')
(removeAll 'removeAll' 'Remove all elements from the playfield')
(reverse 'reverse' 'Reverse the graph')
(roundUpStrays 'roundUpStrays' 'Bring all out-of-container subparts back into view.')
(seesColor: #isOverColor 'whether any part of the object is over the given color')
(show 'show' 'make the object visible')
(shuffleContents 'shuffleContents' 'Shuffle the contents of the playfield')
(stampAndErase 'stampAndErase' 'add my image to the pen trails and go away')
(startAll: 'startAll:' 'start the given script ticking in the object and all of its siblings.')
(startScript: 'start script' 'start the given script ticking')
(stopAll: 'stopAll:' 'make the given script be "normal" in the object and all of its siblings')
(stopScript: 'stop script' 'make the given script be "normal"')
(tellAllSiblings: 'tellAllSiblings:' 'send a message to all siblings')
(touchesA: #touchesA 'whether I touch something that looks like...')
(turn: 'turn by' 'Change the heading of the object by the specified amount')
(unhideHiddenObjects 'unhideHiddenObjects' 'Unhide all hidden objects.')
(wearCostumeOf: 'look like' 'wear the costume of...')
(wrap 'wrap' 'wrap off the edge if appropriate')) language: #YourLanguage.

	self translateCategories: #(
(basic					'basic'					'a few important things')
(#'book navigation'		'book navigation'		'relating to book, stacks, etc')
(button					'button'					'for thinking of this object as a push-button control')
(collections				'collections'				'for thinking of this object as a collection')
(fog					'fog'					'3D fog')
(geometry				'geometry' 				'measurements and coordinates')
(#'color & border'		'color & border'			'matters concerning the colors and borders of objects')
(graphics				'graphics'				'for thinking of this object as a picture')
(#'instance variables'	'instance variables'		'instance variables added by this object')
(joystick				'joystick	'				'the object as a Joystick')
(miscellaneous			'miscellaneous' 			'various commands')
(motion					'motion' 				'matters relating to moving and turning')
(paintbox				'paintbox'				'the painting palette')
(#'pen trails'			'pen trails'				'relating to trails put down by pens')
(#'pen use'				'pen use' 				'use of an object''s "pen"')
(playfield				'playfield'				'the object as a container for other visible objects')
(sampling				'sampling'				'sampling')
(scripts					'scripts'					'methods added by this object')
(slider					'slider'					'functions useful to sliders')
(speaker				'speaker'				'the object as an audio Speaker')
(#'stack navigation'		'stack navigation'		'navigation within a stck')
(storyboard				'storyboard'				'storyboard')
(tests					'tests'					'yes/no tests, to use in "Test" panes of scripts')
(text					'text'					'The object as text')
(viewing				'viewing'				'matters relating to viewing')
(vector					'vector'					'The object as a vector')
 ) language: #YourLanguage.

	self addToTranslationTableFrom: #(
(:						'_'						'assign value')
(Incr:					'increase by'			'increase value by')
(Decr:					'decrease by'			'decrease value by')
(Mult:					'multiply by'			'multiply value by')) language: #YourLanguage
! !


!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'sw 9/10/2001 14:44'!
addCustomCategoriesTo: categoryList
	"Add any further categories to the default list of viewer categories for an object"

	categoryList add: #vector! !

!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 16:34'!
eToyVectorTable
	"Answer a table of specifications to send to #addFromTable: which add the 'players are vectors' extension to the etoy vocabulary."

	"(selector setterOrNil ((arg name  arg type)...) resultType (category ...) 'help msg' 'wording' autoUpdate)"

	^ #(

(+ nil ((aVector  player)) player (geometry) 'Adds two players together, treating each as a vector from the origin.')
(- nil ((aVector  player)) player (geometry) 'Subtracts one player from another, treating each as a vector from the origin.')
(* nil ((aVector  number)) player (geometry) 'Multiply a player by a number, treating the Player as a vector from the origin.')
(/ nil ((aVector  number)) player (geometry) 'Divide a player by a number, treating the Player as a vector from the origin.')

(incr: nil ((aVector  player)) unknown (geometry) 'Each Player is a vector from the origin.  Increase one by the amount of the other.' 'increase by')
(decr: nil ((aVector  player)) unknown (geometry) 'Each Player is a vector from the origin.  Decrease one by the amount of the other.' 'decrease by')
(multBy: nil ((factor  number)) unknown (geometry) 'A Player is a vector from the origin.  Multiply its length by the factor.' 'multiplied by')
(dividedBy: nil ((factor  number)) unknown (geometry) 'A Player is a vector from the origin.  Divide its length by the factor.' 'divided by')

"distance and theta are already in player.  See additionsToViewerCategoryGeometry"
).! !

!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 12:16'!
initialize
	"Initialize the vocabulary"

	super initialize.
	self addFromTable: self eToyVectorTable language: #English.
	self vocabularyName: 'Vector'.
	self documentation: 'This vocabulary adds to the basic etoy experience an interpretation of "players are vectors", requested by Alan Kay and implemented by Ted Kaehler in summer 2001'.
! !

!EToyVectorVocabulary methodsFor: 'method list' stamp: 'sw 9/13/2001 17:26'!
allMethodsInCategory: aCategorySymbol forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the etoy interface which are in the given category, on behalf of anObject, or if it is nil, aClass"

	| likelyReply |
	likelyReply _ super allMethodsInCategory: aCategorySymbol forInstance: anObject ofClass: aClass.
	^ ((anObject isKindOf: Player) and:
		[aCategorySymbol == #vector])
			ifFalse:
				[likelyReply]
			ifTrue:
				[anObject costume class vectorAdditions collect:
					[:anAddition | (self methodInterfaceFrom: anAddition) selector]]! !


!Vocabulary class methodsFor: 'class initialization' stamp: 'sw 9/13/2001 16:09'!
assureNaturalLanguageTranslationsInActiveWorld
	"Assure that the currently-designated natural language is reflected in the tile scripting elements in the active world."

	| languageSymbol |
	languageSymbol _ ActiveWorld currentNaturalLanguage.
	self assureTranslationsAvailableFor: languageSymbol.
	ActiveWorld allTileScriptingElements do:
		[:viewerOrScriptor |
			viewerOrScriptor setNaturalLanguageTo: languageSymbol]

	"Vocabulary assureNaturalLanguageTranslationsInActiveWorld"! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'sw 9/13/2001 16:03'!
initialize
	"Initialize a few standard vocabularies and place them in the AllVocabularies list.  Call this to update all vocabularies."

	self initializeStandardVocabularies.
	self assureNaturalLanguageTranslationsInActiveWorld.

	"Vocabulary initialize"

! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'sw 9/13/2001 14:28'!
initializeStandardVocabularies
	"Initialize a few standard vocabularies and place them in the AllStandardVocabularies list."

	AllStandardVocabularies _ nil.
	self allStandardVocabularies.  "initializes it to an empty dictionary"

	self addEToyVocabulary.
	self addStandardVocabulary: self newPublicVocabulary.
	self addStandardVocabulary: FullVocabulary new.
	self addStandardVocabulary: self newQuadVocabulary.
	self addEToyVectorVocabulary.

	self wonderlandVocabulary.  	"creates it and adds it"
	self numberVocabulary.   		"ditto"
	self vocabularyForClass: Time.   "ditto"
	self vocabularyForClass: String. "ditto"

	"Vocabulary initialize"

! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'sw 9/12/2001 23:06'!
newFullVocabulary
	"Answer an instance of a Full Vocabulary"

	^ FullVocabulary new! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'sw 9/6/2001 15:16'!
quadVocabulary
	"Answer the Quad vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"

	^ self vocabularyNamed: #Quad ifAbsentCreateByCalling: #newQuadVocabulary

	"Vocabulary quadVocabulary"! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'sw 9/6/2001 15:17'!
testVocabulary
	"Answer the Test vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"

	^ self vocabularyNamed: #Test ifAbsentCreateByCalling: #newTestVocabulary

	"Vocabulary testVocabulary"! !

!Vocabulary class methodsFor: 'universal vocabularies' stamp: 'sw 9/6/2001 15:21'!
fullVocabulary
	"Answer the full vocabulary in my AllStandardVocabularies list, creating it if necessary"

	^ self vocabularyNamed: #Full ifAbsentCreateByCalling: #newFullVocabulary! !

!Vocabulary class methodsFor: 'universal vocabularies' stamp: 'sw 9/6/2001 15:17'!
publicVocabulary
	"Answer the public vocabulary, which admits through all non-private categories & selectors"

	^ self vocabularyNamed: #Public ifAbsentCreateByCalling: #newPublicVocabulary! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/6/2001 15:05'!
addStandardVocabulary: aVocabulary
	"Add a vocabulary to the list of standard vocabularies"

	self allStandardVocabularies.  "Assures initialization"
	self removeStandardVocabularyNamed: aVocabulary vocabularyName.
	AllStandardVocabularies at: aVocabulary vocabularyName put: aVocabulary! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/10/2001 14:48'!
allStandardVocabularies
	"Answer a list of the currently-defined vocabularies in my AllStandardVocabularies list"

	AllStandardVocabularies ifNil:
		[AllStandardVocabularies _ IdentityDictionary new].
	^ AllStandardVocabularies values

	"Vocabulary allStandardVocabularies"! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/13/2001 17:32'!
assureTranslationsAvailableFor: aLanguageSymbol
	"Assure that translations are available for the given language.  For economy, they are installed lazily"

	self allStandardVocabularies do:
		[:aVocabulary | aVocabulary assureTranslationsAvailableFor: aLanguageSymbol]! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/13/2001 17:34'!
removeStandardVocabularyNamed: aVocabularyName
	"Remove the vocabulary of the given name from the standard list"

	self allStandardVocabularies.  "Assures initialization"
	AllStandardVocabularies _ AllStandardVocabularies reject:
		[:each | each vocabularyName = aVocabularyName]

	"AllStandardVocabularies _ nil"! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/12/2001 15:07'!
vocabularyNamed: aName
	"Answer the standard vocabulary of the given name, or nil if none found"

	^ self allStandardVocabularies detect: [:aVocabulary | aVocabulary vocabularyName = aName] ifNone: [nil]! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 9/12/2001 15:07'!
vocabularyNamed: aName ifAbsentCreateByCalling: aSelector
	"Answer the standard vocabulary of the given name, creating one if it there is not yet one present by sending aSelector to the receiver"

	| newVocab |
	^ self allStandardVocabularies
		detect:
			[:aVocabulary | aVocabulary vocabularyName = aName]
		ifNone:
			[newVocab _ self perform: aSelector.
			AllStandardVocabularies at: aName put: newVocab.
			newVocab]! !

!Vocabulary class methodsFor: 'eToy vocabularies' stamp: 'sw 9/13/2001 17:29'!
addEToyVectorVocabulary
	"Add the etoy Vector vocabulary to the standard list"

	AllStandardVocabularies at: #Vector put:  EToyVectorVocabulary new

		 ! !

!Vocabulary class methodsFor: 'eToy vocabularies' stamp: 'sw 9/13/2001 17:29'!
addEToyVocabulary
	"Add the etoy vocabulary to the standard list"

	AllStandardVocabularies at: #eToy put: EToyVocabulary new
	
		 ! !

!Vocabulary class methodsFor: 'eToy vocabularies' stamp: 'sw 9/13/2001 17:32'!
eToyVocabulary
	"Answer the etoy vocabulary in the AllStandardVocabularies list, creating it if necessary."

	^ self vocabularyNamed: #eToy ifAbsentCreateByCalling: #addEToyVocabulary! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/13/2001 10:07'!
newWonderlandVocabulary
	"Answer a Wonderland vocabulary -- highly experimental"

	| aVocabulary  |
	"Vocabulary newWonderlandVocabulary"
	self removeStandardVocabularyNamed: #Wonderland.
	aVocabulary _ Vocabulary new vocabularyName: #Wonderland.
	aVocabulary documentation: 'A simple vocabulary for scripting Alice objects'.
	aVocabulary initializeFromTable:  #(
		(color color: () color (basic color) 'The color of the object' unused updating)
		"--"
		(getX setX: () number (basic geometry) 'The x position' unused updating)
		(getY setY: () number (basic geometry) 'The y position' unused updating)
		(getZ setZ: () number (basic geometry) 'The z position' unused updating)
		"--"
		(width setWidth: () number (geometry) 'The width of the object' unused updating)
		(height setHeight: () number (geometry) 'The height of the object' unused updating)
		(depth setDepth: () number (geometry) 'The depth of the object' unused updating)
		"--"
		(heading setHeading: () number (basic geometry) 'The heading of the object' unused updating)
		(forwardBy: unused ((distance number)) none (basic motion) 'Moves the object by the specified distance' 'forward by')
		(turnBy: unused ((angle number)) none (basic motion) 'Turns the object by the specified number of degrees' 'turn by')
		(graphic setGraphic: () graphic (basic graphics) 'The picture currently being worn' unused updating)
		(animationIndex setAnimationIndex: () number (graphics) 'The index in the object''s animation chain' unused updating)
		(emptyScript unused () none (scripts) 'The empty script')
		(distanceToCamera setDistanceToCamera: () number (geometry) 'The distance of the object from the camera' unused updating)
		(distanceTo: unused ((target player)) number (geometry) 'The distance of the object to the given target')
	).
	^ aVocabulary
! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/12/2001 15:06'!
numberVocabulary
	"Answer the standard vocabulary representing numbers, creating it if necessary"

	| aVocab newVocab |
	aVocab _  self allStandardVocabularies detect: [:aVocabulary | 
		aVocabulary vocabularyName = 'Number'] ifNone:
			[self addStandardVocabulary: (newVocab _ self newNumberVocabulary).
			newVocab].
	^ aVocab! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/13/2001 14:28'!
replaceNumberVocabulary
	"Put a new Number vocabulary in place, since the old one was flawed.  No senders, called only by do-its" 
	"Vocabulary replaceNumberVocabulary"

	AllStandardVocabularies removeKey: #Number ifAbsent: [].

	self numberVocabulary! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/12/2001 15:06'!
vocabularyForClass: aClass
	"Answer the standard vocabulary for that class.  Create it if not present and init message exists.  Answer nil if none exists and no init message present."

	| initMsgName aVocab newTypeVocab |
	aVocab _ self allStandardVocabularies
		detect:
			[:aVocabulary | 
				aVocabulary vocabularyName == aClass name]
		ifNone:
			[initMsgName _ 'new', aClass name, 'Vocabulary'.
			newTypeVocab _ nil.
			Symbol hasInterned: initMsgName ifTrue:
				[:symb |
					newTypeVocab _ self perform: symb.
					self addStandardVocabulary: newTypeVocab].
			newTypeVocab].

	^ aVocab! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/6/2001 15:20'!
wonderlandVocabulary
	"Answer the Quad vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"

	^ self vocabularyNamed: #Wonderland ifAbsentCreateByCalling: #newWonderlandVocabulary

	"Vocabulary newWonderlandVocabulary"! !


!WonderlandActor methodsFor: 'eToy-fake' stamp: 'sw 9/13/2001 14:20'!
methodInterfaceForEmptyScript
	"Copied from Player"

	| anInterface |
	anInterface _ MethodInterface new.
	anInterface documentation: 'an empty script; drop on desktop to get a new empty script for this object.'.
	anInterface receiverType: #none.
	anInterface flagAttribute: #scripts.

	anInterface absorbTranslation: 
		(ElementTranslation new wording: #emptyScript helpMessage: 'an empty script; drop on desktop to get a new empty script for this object.' language: #English).

	anInterface selector: #emptyScript type: nil setter: nil.
	^ anInterface! !

!FullVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 23:26'!
rigAFewCategories
	"Formerly used to rig generic categories, now seemingly disfunctional and in abeyance"

	| aMethodCategory |
	true ifTrue: [^ self].

	self flag: #deferred.
"Vocabulary fullVocabulary rigAFewCategories "
	#(	(accessing	'Generally holds methods to read and write instance variables')
		(initialization	'messages typically sent when an object is created, to set up its initial state'))

		do:
			[:pair |
				aMethodCategory _ ElementCategory new categoryName: pair first.
				aMethodCategory documentation: pair second.
				self addCategory: aMethodCategory]! !

Vocabulary class removeSelector: #addGermanVocabulary!
Vocabulary class removeSelector: #addKiswahiliVocabulary!
Vocabulary class removeSelector: #addVocabulary:!
Vocabulary class removeSelector: #allVocabularies!
Vocabulary initialize!
Vocabulary class removeSelector: #kiswahiliVocabulary!
Vocabulary class removeSelector: #methodInterfaceFor:!
Vocabulary class removeSelector: #removeEToyVectorVocabulary!
Vocabulary class removeSelector: #removeVocabularyNamed:!
Vocabulary class removeSelector: #resetMethodInterfaceRepository!
Vocabulary class removeSelector: #storeInterface:forSelector:!


!Vocabulary class reorganize!
('class initialization' assureNaturalLanguageTranslationsInActiveWorld cleanseVocabularyStructures initialize initializeStandardVocabularies)
('queries' instanceWhoRespondsTo:)
('testing and demo' newFullVocabulary newQuadVocabulary newTestVocabulary quadVocabulary testVocabulary)
('universal vocabularies' fullVocabulary newPublicVocabulary publicVocabulary)
('standard vocabulary access' addStandardVocabulary: allStandardVocabularies assureTranslationsAvailableFor: removeStandardVocabularyNamed: vocabularyFrom: vocabularyNamed: vocabularyNamed:ifAbsentCreateByCalling:)
('method interfaces')
('eToy vocabularies' addEToyVectorVocabulary addEToyVocabulary eToyVocabulary)
('type vocabularies' newNumberVocabulary newStringVocabulary newTimeVocabulary newWonderlandVocabulary numberVocabulary replaceNumberVocabulary vocabularyForClass: wonderlandVocabulary)
!


!EToyVocabulary reorganize!
('initialization' addCustomCategoriesTo: addGetterAndSetterInterfacesFromOldSlotSpec: encompassesAPriori: includesSelector:forInstance:ofClass:limitClass: initialize methodInterfaceFrom: morphClassesDeclaringViewerAdditions objectForDataStream: setCategoryDocumentationStrings)
('category list' categoryListForInstance:ofClass:limitClass:)
('method list' allMethodsInCategory:forInstance:ofClass:)
('language translations' addGermanVocabulary addKiswahiliVocabulary assureTranslationsAvailableFor: templateForLanguageTranslation)
!

Vocabulary removeSelector: #addFromTable:!
Vocabulary removeSelector: #addToTranslationTableFrom:!
Vocabulary removeSelector: #symbolWhoseTranslationIs:!
Vocabulary removeSelector: #translateCategories:!
Vocabulary removeSelector: #translateMethodInterfaceWordings:!


!Project reorganize!
('initialization' backgroundColorForMorphicProject backgroundColorForMvcProject defaultBackgroundColor initMorphic initialExtent initialProject initialize installNewDisplay:depth: installPasteUpAsWorld: setChangeSet: setProjectHolder: setServer windowActiveOnFirstClick windowReqNewLabel:)
('accessing' addGuard: addSubProjectNamesTo:indentation: changeSet currentStack: displayDepth: environment findProjectView: forgetExistingURL isCurrentProject isMorphic isTopProject labelString lastDirectory: lastSavedAtSeconds name nextProject parent previousProject projectChangeSet renameTo: setParent: setThumbnail: setViewSize: storeNewPrimaryURL: thumbnail urlList viewSize world)
('menu messages' assureNavigatorPresenceMatchesPreference doWeWantToRename enter enter: enter:revert:saveForRevert: enterAsActiveSubprojectWithin: enterForEmergencyRecovery exit fileOut finalEnterActions finalExitActions installProjectPreferences makeThumbnail saveProjectPreferences saveState validateProjectNameIfOK: viewLocFor:)
('release' addDependent: canDiscardEdits deletingProject: okToChange release removeChangeSetIfPossible subProjects)
('printing' printOn:)
('file in/out' armsLengthCommand:withDescription: assureIntegerVersion beep bumpVersion: compressFilesIn:to:in:resources: couldBeSwappedOut currentVersionNumber decideAboutCreatingBlank: doArmsLengthCommand: downloadUrl ensureChangeSetNameUnique exportSegmentFileName:directory: exportSegmentWithCatagories:classes:fileName:directory: exportSegmentWithChangeSet:fileName:directory: findAFolderToLoadProjectFrom findAFolderToStoreProjectIn fromMyServerLoad: hasBadNameForStoring htmlPagePrototype loadFromServer loadFromServer: objectForDataStream: primaryServer primaryServerIfNil: projectExtension restoreReferences revert saveAs saveForRevert serverList squeakletDirectory storeDataOn: storeHtmlPageIn: storeOnServer storeOnServerAssumingNameValid storeOnServerInnards storeOnServerShowProgressOn:forgetURL: storeOnServerWithProgressInfo storeOnServerWithProgressInfoOn: storeSegment storeSegmentNoFile storeSomeSegment storeToMakeRoom tryToFindAServerWithMe url urlForLoading versionForFileName versionFrom: versionedFileName writeFileNamed:fromDirectory:toServer: writeStackText:in:registerIn:)
('object fileIn' convertToCurrentVersion:refStream:)
('project parameters' currentStack initializeProjectParameters initializeProjectPreferences noteThatParameter:justChangedTo: parameterAt: parameterAt:ifAbsent: projectParameterAt: projectParameterAt:default: projectParameterAt:ifAbsent: projectParameterAt:put: projectParameters projectPreferenceAt: projectPreferenceAt:ifAbsent: projectPreferenceFlagDictionary rawParameters removeParameter:)
('flaps support' assureFlapIntegrity cleanseDisabledGlobalFlapIDsList enableDisableGlobalFlap: flapsSuppressed flapsSuppressed: globalFlapEnabledString: globalFlapWithIDEnabledString: isFlapEnabled: isFlapIDEnabled: showSharedFlaps)
('language' chooseNaturalLanguage naturalLanguage setNaturalLanguageTo: stringForLanguageNameIs:)
('displaying' createViewIfAppropriate displayZoom: imageForm imageFormOfSize:depth: showZoom)
('isolation layers' beIsolated compileAll:from: compileAllIsolated:from: invoke invokeFrom: isIsolated isolationHead isolationSet layersToTop propagateChanges revoke)
('isolated changes')
('OBSOLETE' obsolete)
('SuperSwiki' tellAFriend tellAFriend:)
('resources' abortResourceLoading resourceDirectoryName resourceManager resourceManager: resourceUrl startResourceLoading storeResourceList:in:)
!

Preferences class removeSelector: #playersAreVectors!
Preferences class removeSelector: #playersAreVectorsToggled!
MethodInterface removeSelector: #elementWording:!
MethodInterface removeSelector: #wording:selector:type:setter:!

!ObjectWithDocumentation reorganize!
('initialization' initialize)
('documentation' documentation: documentationOrNil editDescription)
('queries' documentation wording)
('translation' absorbTranslation: translatedToPrevailingLanguage)
('miscellaneous' elementSymbol)
!

ElementTranslation class removeSelector: #fromPair:!
ElementTranslation removeSelector: #wording:helpMessage:!

Object subclass: #ObjectWithDocumentation
	instanceVariableNames: 'authoringStamp properties elementSymbol naturalLanguageTranslations '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Protocols'!

ObjectWithDocumentation subclass: #MethodInterface
	instanceVariableNames: 'selector argumentVariables resultSpecification receiverType attributeKeywords defaultStatus '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Protocols'!

ObjectWithDocumentation subclass: #Vocabulary
	instanceVariableNames: 'vocabularyName categories methodInterfaces object limitClass translationTable '
	classVariableNames: ' AllStandardVocabularies  '
	poolDictionaries: ''
	category: 'System-Protocols'!

"Postscript:"
Preferences setParameter: #systemLanguage to: #English.
Preferences expungePreferenceNamed: #playersAreVectors.
Utilities appendToCommonRequests: '-'.
Utilities appendToCommonRequests: 'ActiveWorld installVectorVocabulary.'.
Utilities appendToCommonRequests: 'ActiveWorld abandonVocabularyPreference.'.
Vocabulary initialize.
!

