'From Squeak 2.3 of January 14, 1999 on 8 March 1999 at 10:42:32 pm'!
"Change Set:		windowCopyFixup-sw
Date:			6 March 1999
Author:			Scott Wallace

Adds change sorters to the kinds of system windows made available in parts bins and parts flaps.

Fixes up veryDeepCopies of various system windows that need to be updated to conform to local circumstances when dragged out of a parts bin -- applies to browsers (category list may have changed), file lists (directories may have changed), and change sorters (change sets may have changed).

Also, sets things so that windows in the parts flap and in the standard parts bin will open up to their standard default sizees."!


!Browser methodsFor: 'initialize-release' stamp: 'sw 3/6/1999 11:17'!
veryDeepFixupWith: deepCopier
	super veryDeepFixupWith: deepCopier.
	self changed: #systemCategoryList! !


!ChangeSet methodsFor: 'initialize-release' stamp: 'sw 3/6/1999 09:31'!
veryDeepCopyWith: deepCopier
	"Return self; this is NOT the way to launch new change sets!! Having this method here allows Change Sorters to be in parts bins"! !


!ChangeSorter methodsFor: 'creation' stamp: 'sw 3/6/1999 09:33'!
morphicWindow
	"ChangeSorter new openAsMorph"
	|  window |

	myChangeSet ifNil: [self myChangeSet: Smalltalk changes]. 
	window _ (SystemWindow labelled: self labelString) model: self.
	self openAsMorphIn: window rect: (0@0 extent: 1@1).
	^ window
! !

!ChangeSorter methodsFor: 'creation' stamp: 'sw 3/6/1999 09:34'!
openAsMorph
	"ChangeSorter new openAsMorph"
	^ self morphicWindow openInWorld.
! !

!ChangeSorter methodsFor: 'creation' stamp: 'sw 3/6/1999 11:14'!
veryDeepFixupWith: deepCopier
	super veryDeepFixupWith: deepCopier.
	self updateIfNecessary! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/6/1999 11:01'!
updateIfNecessary
	"recompute all of my panes"
	| newList |
	myChangeSet isMoribund ifTrue:
		[^ self showChangeSet: Smalltalk changes].

	self okToChange ifFalse: [^ self].
	priorChangeSetList == nil
		ifTrue: [priorChangeSetList _ self changeSetList.
				self changed: #changeSetList]
		ifFalse: [newList _ self changeSetList.
				priorChangeSetList = newList ifFalse:
					[priorChangeSetList _ newList.
					self changed: #changeSetList]].
	self showChangeSet: myChangeSet! !


!DualChangeSorter methodsFor: 'all' stamp: 'sw 3/6/1999 09:34'!
morphicWindow
	
	| window |
	leftCngSorter _ ChangeSorter new myChangeSet: Smalltalk changes.
	leftCngSorter parent: self.
	rightCngSorter _ ChangeSorter new myChangeSet: 
			ChangeSorter secondaryChangeSet.
	rightCngSorter parent: self.

	window _ (SystemWindow labelled: leftCngSorter label) model: self.
	"topView minimumSize: 300 @ 200."
	leftCngSorter openAsMorphIn: window rect: (0@0 extent: 0.5@1).
	rightCngSorter openAsMorphIn: window rect: (0.5@0 extent: 0.5@1).
	^ window
! !

!DualChangeSorter methodsFor: 'all' stamp: 'sw 3/6/1999 09:34'!
openAsMorph
	^ self morphicWindow openInWorld
! !


!FileList methodsFor: 'volume list and pattern' stamp: 'sw 3/6/1999 11:39'!
veryDeepFixupWith: deepCopier
	super veryDeepFixupWith: deepCopier.
	volListIndex _ 1.
	self directory: FileDirectory default.
	self updateFileList! !


!Presenter methodsFor: 'palette & parts bin' stamp: 'sw 3/6/1999 11:57'!
customPagesForPartsBin
	| aPage aWindow  pu pageList aClass |
	pageList _ OrderedCollection new.

	pageList add: (aPage _ self newPageForStandardPartsBin).
	aPage addMorphBack: (PasteUpMorph authoringPrototype color: (Color r: 0.96 g: 0.96 b: 0.96)).
	aPage addMorphBack: TabbedPalette authoringPrototype.
	aPage addMorphBack: BookMorph authoringPrototype.
	aPage addMorphBack: Morph new previousPageButton markAsPartsDonor.
	aPage addMorphBack: Morph new nextPageButton markAsPartsDonor.
	#(DatumMorph) do:
		[:aName | (aClass _ Smalltalk at: aName ifAbsent: [nil]) ifNotNil:
			[aPage addMorphBack: aClass authoringPrototype]].

	pageList add: (aPage _ self newPageForStandardPartsBin).
	aWindow _ Browser new openAsMorphEditing: nil.
	aWindow setLabel: 'System Browser'.
	aPage addMorphBack: aWindow applyModelExtent.
	aWindow _ Workspace new embeddedInMorphicWindowLabeled: 'Workspace'.
	aPage addMorphBack: aWindow applyModelExtent.
	aPage addMorphBack: FileList openAsMorph applyModelExtent.

	aPage addMorphBack: DualChangeSorter new morphicWindow applyModelExtent.
	aPage addMorphBack: ChangeSorter new morphicWindow applyModelExtent.

	aWindow _ SelectorBrowser new morphicWindow.
	aWindow setLabel: 'Selector Browser'.
	aPage addMorphBack: aWindow.
	aPage addMorphBack: ((pu _ PasteUpMorph newSticky borderInset) embeddedInMorphicWindowLabeled: 'assembly').

	pageList add: (aPage _ self newPageForStandardPartsBin).
	aPage addMorphBack: ScriptingSystem newScriptingSpace.
	aPage addMorphBack: ScriptingSystem scriptControlButtons.
	aPage addMorphBack: TrashCanMorph new.
	aPage addMorphBack: PasteUpMorph authoringPrototype.
	aPage addMorphBack: ((pu _ PasteUpMorph newSticky) embeddedInMorphicWindowLabeled: 'assembly').
	pu color: (Color r: 0.839 g: 1.0 b: 0.935).

	pageList do:
		[:page |
			page  replaceTallSubmorphsByThumbnails.
			page fixLayout].

	^ pageList
! !


!SystemWindow methodsFor: 'initialization' stamp: 'sw 3/6/1999 10:52'!
applyModelExtent
	self extent: model initialExtent! !


