'From Squeak 2.3 of January 14, 1999 on 23 March 1999 at 2:18:15 pm'!
"Change Set:		trashFromHalo-sw
Date:			23 March 1999
Author:			Scott Wallace

When the user hits the 'dismiss' halo handle, move the behaloed object to the trash rather than just clobbering it.

The 'dismiss' halo handle is made pink-with-an-x, like various other go-away controls elsewhere in morphic, and it is given a natural button-like action when moused in, rather than irreversibly activating on the initial mouse down.

When the Trash is open for manipulation, it now sports a button allowing you to empty it.

The header of the open Trash window now reports on how many pages of trash there are.

Along the way, fixes a long-standing bug in Morph.replaceSubmorph:by:, which, while remembering to start stepping the new object itself, did not start stepping any submorphs which it in turn might have."!


!Morph methodsFor: 'accessing' stamp: 'sw 3/22/1999 16:22'!
balloonHelpTextForHandle: aHandle
	|  itsSelector |
	itsSelector _ aHandle eventHandler firstMouseSelector.
	#(	(startRot:with:				'Rotate')
		(startGrow:with:				'Change size') 
		(doDup:with:				'Duplicate')
		(maybeDoDup:with:			'Duplicate')
		(doGrab:with:				'Pick up')
		(startDrag:with:				'Move')
		(doMenu:with:				'Menu')
		(doDebug:with:				'Debug')
		(mouseDownOnHelpHandle:	'Help')
		(tearOffTile					'Make a Tile')
		(paintBackground			'Paint background')
		(openViewerForArgument	'View me')
		(dismiss						'Remove')
		(mouseDownInDimissHandle:with:	'Move to trash')
		(editDrawing				'Repaint')
		(startScale:with:				'Change scale') 
		(chooseFont					'Change font')
		(chooseStyle					'Change style')
		(chooseEmphasis				'Change emphasis')
		(makeNewDrawingWithin	'Paint new object'))  "This last one not currently in use"


	do:
		[:pair | itsSelector == pair first ifTrue: [^ pair last]].
	^ 'unknown halo handle'! !

!Morph methodsFor: 'submorphs-add/remove' stamp: 'sw 3/23/1999 12:22'!
replaceSubmorph: oldMorph by: newMorph
	| index itsPosition w |
	oldMorph stopStepping.
	itsPosition _ oldMorph position.
	index _ submorphs indexOf: oldMorph.
	oldMorph privateDelete.
	self privateAddMorph: newMorph atIndex: index.
	newMorph position: itsPosition.
	(w _ newMorph world) ifNotNil:
		[w startSteppingSubmorphsOf: newMorph]! !


!HaloMorph methodsFor: 'private' stamp: 'sw 3/22/1999 21:28'!
addHandles
	| box s dismissHandle lab |
	target isWorldMorph ifTrue: [^ self addHandlesForWorldHalos].
	self removeAllMorphs.  "remove old handles, if any"
	s _ self handleSize.
	self bounds: target renderedMorph fullBoundsInWorld.  "update my size"
	box _ (self bounds expandBy: s+1)
			intersect: (self world bounds insetBy: 5@5).

	self addHandleAt: box topLeft color: Color red
		on: #mouseDown send: #doMenu:with: to: self.

	self addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #tearOffTile to: innerTarget.

	dismissHandle _ self addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color red muchLighter
		on: #mouseDown send: #mouseDownInDimissHandle:with: to: self.
	dismissHandle on: #mouseUp send: #maybeDismiss:with: to: self.
	dismissHandle on: #mouseStillDown send: #setDismissColor:with: to: self.
	dismissHandle addMorphBack: (lab _ StringMorph contents: 'X' font: ScriptingSystem fontForScriptorButtons).
	lab position: dismissHandle position + (4 @ 0).
	lab lock.

	self addHandleAt: box leftCenter color: Color cyan
		on: #mouseDown send: #openViewerForArgument to: innerTarget.

	self addHandleAt: box topCenter color: Color black
		on: #mouseDown send: #doGrab:with: to: self.

	(self addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown
		on: #mouseDown send: #startDrag:with: to: self)
		on: #mouseStillDown send: #doDrag:with: to: self.

	self addHandleAt: box topRight color: Color green
		on: #mouseDown send: #doDup:with: to: self.

	Preferences showDebugHaloHandle ifTrue:
		[self addHandleAt: (box rightCenter + (0 @ ((s+2) negated)) min: box rightCenter + box bottomRight // 2)
			color: Color blue veryMuchLighter
			on: #mouseDown send: #doDebug:with: to: self].

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue
			on: #mouseDown send: #mouseDownOnHelpHandle: to: innerTarget)
			on: #mouseUp send: #deleteBalloon to: innerTarget].

	(self addHandleAt: box bottomLeft color: Color blue
		on: #mouseDown send: #startRot:with: to: self)
		on: #mouseStillDown send: #doRot:with: to: self.

	target isFlexMorph
		ifTrue: [(self addHandleAt: box bottomRight color: Color lightOrange
				on: #mouseDown send: #startScale:with: to: self)
				on: #mouseStillDown send: #doScale:with: to: self]
		ifFalse: [(self addHandleAt: box bottomRight color: Color yellow
				on: #mouseDown send: #startGrow:with: to: self)
				on: #mouseStillDown send: #doGrow:with: to: self].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: box string: target externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !

!HaloMorph methodsFor: 'private' stamp: 'sw 3/22/1999 15:45'!
dismiss
	"Remove my target from the world."

	| w |
	w _ self world.
	w ifNotNil: [w stopStepping: target].
	self delete.
	TrashCanMorph moveToTrash: target! !

!HaloMorph methodsFor: 'private' stamp: 'sw 3/22/1999 16:13'!
maybeDismiss: evt with: dismissHandle
	"Ask hand to dismiss my target if mouse comes up in it."

	(dismissHandle containsPoint: evt cursorPoint)
		ifFalse:
			[self delete.
			target addHalo]
		ifTrue:
			[self delete.
			TrashCanMorph moveToTrash: target]! !

!HaloMorph methodsFor: 'private' stamp: 'sw 3/22/1999 16:22'!
mouseDownInDimissHandle: evt with: dismissHandle
	Preferences soundsEnabled ifTrue: [TrashCanMorph playMouseEnterSound].
	self removeAllHandlesBut: dismissHandle.
	dismissHandle color: Color darkGray.! !

!HaloMorph methodsFor: 'private' stamp: 'sw 3/22/1999 21:26'!
setDismissColor: evt with: dismissHandle
	"Called on mouseStillDown in the dismiss handle; set the color appropriately."

	| colorToUse |
	colorToUse _  (dismissHandle containsPoint: evt cursorPoint)
		ifFalse:
			[Color red muchLighter]
		ifTrue:
			[Color lightGray].
	dismissHandle color: colorToUse! !


!TrashCanMorph class methodsFor: 'all' stamp: 'sw 3/22/1999 15:44'!
moveToTrash: aMorph
	| palette |
	Preferences soundsEnabled ifTrue: [self playDeleteSound].
	palette _ aMorph standardPalette.
	aMorph delete.
	(palette notNil and: [palette isInWorld])
		ifTrue:
			[palette addToTrash: aMorph]
		ifFalse:
			[aMorph == Utilities scrapsBook ifFalse:
				[Utilities addToTrash: aMorph]]! !


!Utilities class methodsFor: 'scraps' stamp: 'sw 3/23/1999 09:18'!
maybeEmptyTrash
	(self confirm: 'Do you really want to empty the trash?')
		ifTrue: [self emptyScrapsBook]! !

!Utilities class methodsFor: 'scraps' stamp: 'sw 3/23/1999 11:54'!
scrapsBook
	| header aButton label |
	ScrapsBook ifNil:
		[ScrapsBook _ BookMorph new pageSize: 200@300; setNameTo: 'scraps'.
		ScrapsBook color: Color yellow muchLighter.
		ScrapsBook borderColor: Color darkGray; borderWidth: 2.
		ScrapsBook removeEverything; showPageControls; insertPage.
		header _ AlignmentMorph newRow centering: #center.
		header setProperty: #header toValue: true.
		header addMorph: (aButton _ SimpleButtonMorph new label: 'X' font: ScriptingSystem fontForScriptorButtons).
		aButton target: ScrapsBook; color:  Color lightRed; actionSelector: #delete;
				setBalloonText: 'Remove
(to view again later, click on any trashcan).'.

		header addMorphBack: AlignmentMorph newVariableTransparentSpacer beSticky.
		header addMorphBack: 	(label _ UpdatingStringMorph new target: self) beSticky.
		label getSelector: #trashTitle; useStringFormat; step.
		header addMorphBack: AlignmentMorph newVariableTransparentSpacer beSticky.
		header addMorphBack: (aButton _ SimpleButtonMorph new label: 'E' font: ScriptingSystem fontForScriptorButtons).
		aButton target: Utilities; color:  Color veryLightGray; actionSelector: #maybeEmptyTrash;
				setBalloonText: 'Click here to empty the trash.'.
		ScrapsBook currentPage addMorph: (TextMorph new contents: 'Objects you drag into the trash will automatically be saved here, one object per page, in case you need them later.

You can always open up this book by clicking on a trash can.

You can retrieve objects from the trash just by dragging them out from here.

You can individually expunge objects by hitting the - control, and you can empty out all the objects in the trash can by hitting the "E" button at top right.'
			wrappedTo: 190).

		ScrapsBook addMorphFront: header.
		ScrapsBook setProperty: #scraps toValue: true].
	^ ScrapsBook

	"Utilities emptyScrapsBook"
! !

!Utilities class methodsFor: 'scraps' stamp: 'sw 3/23/1999 12:23'!
trashTitle
	| label pgs |
	label _ 'T R A S H'.
	^ (pgs _ ScrapsBook pages size) < 2
		ifTrue:
			[label]
		ifFalse:
			[label, '  (', pgs printString, ' pages)']! !


"Postscript:
"
Utilities emptyScrapsBook.
!

