'From Squeak3.7gamma of ''17 July 2004'' [latest update: #5978] on 3 August 2004 at 1:31:40 am'!
"Change Set:		SysWinInitLabelArea-dew
Date:			29 July 2004
Author:			Doug Way

Simple cut & paste refactoring which splits out an initializeLabelArea method called from SystemWindow>>initialze.  This makes regenerating label areas (titlebars) on existing windows (e.g. for the gamma image preparation) doable.

Also fixes centering of label in the titlebar, and only uses bold with non-TT fonts for the window title (since the bold TT fonts are quite a bit thicker/wider than bold strikefonts)."!


!SystemWindow methodsFor: 'initialization' stamp: 'dew 7/29/2004 00:31'!
initialize
	"Initialize a system window. Add label, stripes, etc., if desired"
	super initialize.
	allowReframeHandles := true.
	labelString
		ifNil: [labelString := 'Untitled Window' translated].
	isCollapsed := false.
	activeOnlyOnTop := true.
	paneMorphs := Array new.
	Preferences alternativeWindowLook
		ifTrue: [""
			borderColor := #raised.
			borderWidth := 2.
			color := Color white]
		ifFalse: [""
			borderColor := Color black.
			borderWidth := 1.
			color := Color black].
	self layoutPolicy: ProportionalLayout new.
	self wantsLabel
		ifTrue: [self initializeLabelArea].
	self
		on: #mouseEnter
		send: #spawnReframeHandle:
		to: self.
	self
		on: #mouseLeave
		send: #spawnReframeHandle:
		to: self.
	self extent: 300 @ 200.
	mustNotClose := false.
	updatablePanes := Array new! !

!SystemWindow methodsFor: 'initialization' stamp: 'dew 8/3/2004 01:21'!
initializeLabelArea
	"Initialize the label area (titlebar) for the window."
	label := StringMorph new
				contents: labelString;
				font: Preferences windowTitleFont
				emphasis: (Preferences windowTitleFont isTTCFont ifTrue: [0] ifFalse: [1]).
	"Add collapse box so #labelHeight will work"
	collapseBox := self createCollapseBox.
	stripes := Array
				with: (RectangleMorph newBounds: bounds)
				with: (RectangleMorph newBounds: bounds).
	"see extent:"
	self addLabelArea.
	labelArea
		addMorph: (stripes first borderWidth: 1).
	labelArea
		addMorph: (stripes second borderWidth: 2).
	self setLabelWidgetAllowance.
	self addCloseBox.
	self addMenuControl.
	labelArea addMorph: label.
	self wantsExpandBox
		ifTrue: [self addExpandBox].
	labelArea addMorph: collapseBox.
	self setFramesForLabelArea.
	Preferences clickOnLabelToEdit
		ifTrue: [label
				on: #mouseDown
				send: #relabel
				to: self].
	Preferences noviceMode
		ifTrue: [closeBox
				ifNotNil: [closeBox setBalloonText: 'close window' translated].
			menuBox
				ifNotNil: [menuBox setBalloonText: 'window menu' translated].
			collapseBox
				ifNotNil: [collapseBox setBalloonText: 'collapse/expand window' translated]].
! !

!SystemWindow methodsFor: 'initialization' stamp: 'dew 8/3/2004 01:11'!
setFramesForLabelArea
	"an aid to converting old instances, but then I found  
	convertAlignment (jesse welton's note)"
	| frame |
	labelArea
		ifNil: [^ self].
	frame := LayoutFrame new.
	frame leftFraction: 0.5;
		 topFraction: 0.5;
		 leftOffset: label width negated // 2;
		 topOffset: label height negated // 2.
	label layoutFrame: frame.
	frame := LayoutFrame new.
	frame rightFraction: 1;
		 topFraction: 0;
		 rightOffset: -2;
		 topOffset: 0.
	collapseBox
		ifNotNilDo: [:cb | cb layoutFrame: frame].
	stripes isEmptyOrNil
		ifFalse: [frame := LayoutFrame new.
			frame leftFraction: 0;
				 topFraction: 0;
				 rightFraction: 1;
				 leftOffset: 1;
				 topOffset: 1;
				 rightOffset: -1;
				 bottomOffset: -2.
			stripes first layoutFrame: frame.
			stripes first height: self labelHeight - 1.
			stripes first hResizing: #spaceFill.
			frame := LayoutFrame new.
			frame leftFraction: 0;
				 topFraction: 0;
				 rightFraction: 1;
				 leftOffset: 3;
				 topOffset: 3;
				 rightOffset: -3.
			stripes last layoutFrame: frame.
			stripes last height: self labelHeight - 5.
			stripes last hResizing: #spaceFill].
	labelArea
		ifNotNil: [
			frame := LayoutFrame fractions: (0@0 corner: 1@0) offsets: ((1@self labelHeight negated - 1) corner: 0@0  ).
			Preferences alternativeWindowLook
				ifTrue: [frame leftOffset: 0].
			labelArea layoutFrame: frame]! !

!SystemWindow methodsFor: 'label' stamp: 'dew 8/3/2004 01:12'!
setLabel: aString
	| frame |
	labelString _ aString.
	label ifNil: [^ self].
	label contents: aString.
	self labelWidgetAllowance.  "Sets it if not already"
	self isCollapsed
		ifTrue: [self extent: (label width + labelWidgetAllowance) @ (self labelHeight + 2)]
		ifFalse: [label fitContents; setWidth: (label width min: bounds width - labelWidgetAllowance).
				label align: label bounds topCenter with: bounds topCenter + (0@borderWidth).
				collapsedFrame ifNotNil:
					[collapsedFrame _ collapsedFrame withWidth: label width + labelWidgetAllowance]].
	frame _ LayoutFrame new.
	frame leftFraction: 0.5;
		 topFraction: 0.5;
		 leftOffset: label width negated // 2;
		 topOffset: label height negated // 2.
	label layoutFrame: frame.
! !

